import os

from turcar.common import path_startswith


def test_path_startswith():
    assert path_startswith("/kala/pala", "/kala")
    assert path_startswith("/kala/pala", "/kala/")

    assert not path_startswith("/kala/pala", "/pala")
    assert not path_startswith("/kalapala/pala", "/kala")

    if os.name == "nt":
        assert path_startswith("/kala/pala", "/KALA")
    else:
        assert not path_startswith("/kala/pala", "/KALA")

    if os.name == "nt":
        assert path_startswith("C:\\foo\\bar", "C:\\foo")
        assert path_startswith("C:\\foo", "C:\\")
        assert path_startswith("C:/foo", "C:\\")
        assert path_startswith("C:\\foo", "C:/")

        assert path_startswith("C:\\foo\\dir\\file", "C:\\foo")
        assert path_startswith("C:/foo\\dir\\file", "c:\\FOO")
        assert path_startswith("c:\\FOO\\dir\\file", "C:/foo")

        assert path_startswith("C:\\FOO\\dir\\dir2\\..\\ee", "C:/foo/dir/ee")

        assert path_startswith("c:\\foo\\bar.txt/kala\\pala", "C:\\foo\\bar.txt/kala")
        assert path_startswith("c:\\foo\\bar.txt/kala\\pala", "C:\\")

        assert not path_startswith("C:\\kalapala\\pala", "C:\\kala")



from pybricks.parameters import Port


class TouchSensor:
    """
    LEGO® MINDSTORMS® NXT Touch Sensor.

    Args:
        port (Port): Port to which the sensor is connected.
    """

    def __init__(self, port: Port):
        if port == Port.A or port == Port.B or port == Port.C or port == Port.D:
            raise ValueError("Sensors must use Port S1, S2, S3, or S4.")

    def pressed(self) -> bool:
        """
        Checks if the sensor is pressed.

        Returns:
            True if the sensor is pressed, False if it is not pressed.
        """
        return False



import argparse
import sys
from typing import Any, List, Optional

from pipkin import __version__


def parse_arguments(raw_args: Optional[List[str]] = None) -> Any:
    if raw_args is None:
        raw_args = sys.argv[1:]

    main_parser = argparse.ArgumentParser(
        description="Tool for managing MicroPython and CircuitPython packages",
        allow_abbrev=False,
        add_help=False,
    )

    general_group = main_parser.add_argument_group(title="general")

    general_group.add_argument(
        "-h",
        "--help",
        help="Show this help message and exit",
        action="help",
    )
    general_group.add_argument(
        "-V",
        "--version",
        help="Show program version and exit",
        action="version",
        version=__version__,
    )
    verbosity_group = general_group.add_mutually_exclusive_group()
    verbosity_group.add_argument(
        "-v",
        "--verbose",
        help="Show more details about the process",
        action="store_true",
    )
    verbosity_group.add_argument(
        "-q",
        "--quiet",
        help="Don't show non-error output",
        action="store_true",
    )

    connection_group = main_parser.add_argument_group(
        title="target selection (pick one or let pipkin autodetect the port or mount)"
    )
    connection_exclusive_group = connection_group.add_mutually_exclusive_group()

    connection_exclusive_group.add_argument(
        "-p",
        "--port",
        help="Serial port of the target device",
        metavar="<port>",
    )
    connection_exclusive_group.add_argument(
        "-m",
        "--mount",
        help="Mount point (volume, disk, drive) of the target device",
        metavar="<path>",
    )
    connection_exclusive_group.add_argument(
        "-d",
        "--dir",
        help="Directory in the local filesystem",
        metavar="<path>",
    )
    # connection_exclusive_group.add_argument(
    #     "-e",
    #     "--exe",
    #     help="Interpreter executable (Unix or Windows port)",
    #     metavar="<path>",
    # )

    # sub-parsers
    subparsers = main_parser.add_subparsers(
        title="commands",
        description='Use "pipkin <command> -h" for usage help of a command ',
        dest="command",
        required=True,
    )

    install_parser = subparsers.add_parser(
        "install",
        help="Install packages.",
        description="Installs upip or pip compatible distribution packages onto "
        "a MicroPython/CircuitPython device  or into a local directory.",
    )

    uninstall_parser = subparsers.add_parser("uninstall", help="Uninstall packages.")
    list_parser = subparsers.add_parser("list", help="List installed packages.")
    show_parser = subparsers.add_parser(
        "show", help="Show information about one or more installed packages."
    )
    freeze_parser = subparsers.add_parser(
        "freeze", help="Output installed packages in requirements format."
    )
    _check_parser = subparsers.add_parser(
        "check", help="Verify installed packages have compatible dependencies."
    )
    download_parser = subparsers.add_parser("download", help="Download packages.")
    wheel_parser = subparsers.add_parser(
        "wheel", help="Build Wheel archives for your requirements and dependencies."
    )
    cache_parser = subparsers.add_parser("cache", help="Inspect and manage pipkin cache.")

    # common options
    for parser in [install_parser, download_parser, wheel_parser]:
        parser.add_argument(
            "specs",
            help="Package specification, eg. 'micropython-os' or 'micropython-os>=0.6'",
            nargs="*",
            metavar="<spec>",
        )

        specs_group = parser.add_argument_group(title="package selection")

        specs_group.add_argument(
            "-r",
            "--requirement",
            help="Install from the given requirements file.",
            nargs="*",
            dest="requirement_files",
            metavar="<file>",
            default=[],
        )
        specs_group.add_argument(
            "-c",
            "--constraint",
            help="Constrain versions using the given constraints file.",
            nargs="*",
            dest="constraint_files",
            metavar="<file>",
            default=[],
        )
        specs_group.add_argument(
            "--no-deps",
            help="Don't install package dependencies.",
            action="store_true",
        )
        specs_group.add_argument(
            "--pre",
            help="Include pre-release and development versions. By default, pipkin only finds stable versions.",
            action="store_true",
        )

    # index-related
    for parser in [install_parser, download_parser, wheel_parser, list_parser]:
        index_group = parser.add_argument_group(title="index selection")
        index_group.add_argument(
            "-i",
            "--index-url",
            help="Base URL of the Python Package Index (default https://pypi.org/simple).",
            metavar="<url>",
        )
        index_group.add_argument(
            "--extra-index-url",
            help="Extra URLs of package indexes to use in addition to --index-url.",
            nargs="*",
            dest="extra_index_urls",
            default=[],
            metavar="<url>",
        )
        index_group.add_argument(
            "--no-index",
            help="Ignore package index (only looking at --find-links URLs instead).",
            action="store_true",
        )
        index_group.add_argument(
            "--no-mp-org",
            help="Don't let micropython.org/pi override other indexes.",
            action="store_true",
        )
        index_group.add_argument(
            "-f",
            "--find-links",
            help="If a URL or path to an html file, then parse for links to archives such as sdist "
            "(.tar.gz) or wheel (.whl) files. If a local path or "
            "file:// URL that's a directory, then look for archives in the directory listing.",
            metavar="<url|file|dir>",
        )

    for parser in [uninstall_parser, show_parser]:
        parser.add_argument(
            "packages",
            help="Package name",
            nargs="*",
            metavar="<name>",
        )

    for parser in [list_parser, freeze_parser]:
        # parser.add_argument(
        #     "--user",
        #     help="Only output packages installed in user-site. Relevant with Unix and Windows ports",
        #     action="store_true",
        # )
        # parser.add_argument(
        #     "--path",
        #     help="Restrict to the specified installation path for listing packages.",
        #     nargs="*",
        #     dest="paths",
        #     metavar="<path>",
        #     default=[],
        # )
        parser.add_argument(
            "--exclude",
            help="Exclude specified package from the output.",
            nargs="*",
            dest="excludes",
            metavar="<package>",
            default=[],
        )

    # install_parser.add_argument(
    #     "-t",
    #     "--target",
    #     help="Target directory in the target filesystem (eg. /lib)",
    #     metavar="<dir>",
    # )
    # install_parser.add_argument(
    #     "--user",
    #     help="Install to the Python user install directory for target platform. "
    #     "Only relevant with Unix and Windows ports",
    #     action="store_true",
    # )
    install_parser.add_argument(
        "-U",
        "--upgrade",
        help="Upgrade all specified packages to the newest available version. "
        "The handling of dependencies depends on the upgrade-strategy used.",
        action="store_true",
    )
    install_parser.add_argument(
        "--upgrade-strategy",
        help="Determines how dependency upgrading should be handled [default: only-if-needed].\n"
        "'eager' - dependencies are upgraded regardless of whether the currently installed "
        "version satisfies the requirements of the upgraded package(s).\n"
        "'only-if-needed' - are upgraded only when they do not satisfy the requirements of the "
        "upgraded package(s).",
        choices=["only-if-needed", "eager"],
        default="only-if-needed",
        metavar="<upgrade_strategy>",
    )
    install_parser.add_argument(
        "--force-reinstall",
        help="Reinstall all packages even if they are already up-to-date.",
        action="store_true",
    )
    install_parser.add_argument(
        "--compile",
        help="Compile and install mpy files.",
        action="store_true",
    )

    uninstall_parser.add_argument(
        "-r",
        "--requirement",
        help="Uninstall all the packages listed in the given requirements file.",
        nargs="*",
        dest="requirement_files",
        metavar="<file>",
        default=[],
    )

    uninstall_parser.add_argument(
        "-y",
        "--yes",
        help="Don't ask for confirmation of uninstall deletions.",
        action="store_true",
    )

    list_parser.add_argument(
        "-o",
        "--outdated",
        help="List outdated packages",
        action="store_true",
    )
    list_parser.add_argument(
        "-u",
        "--uptodate",
        help="List uptodate packages",
        action="store_true",
    )
    list_parser.add_argument(
        "--pre",
        help="Also consider pre-release and development versions when deciding whether package is outdated or uptodate.",
        action="store_true",
    )
    list_parser.add_argument(
        "--not-required",
        help="List packages that are not dependencies of installed packages.",
        action="store_true",
    )
    list_parser.add_argument(
        "--format",
        help="Select the output format among: columns (default), freeze, or json",
        choices=["columns", "freeze", "json"],
        default="columns",
        metavar="<list_format>",
    )

    download_parser.add_argument(
        "-d",
        "--dest",
        help="Download packages into <dir>. Default: current directory.",
        default=".",
        metavar="<dir>",
    )

    wheel_parser.add_argument(
        "-w",
        "--wheel-dir",
        help="Build wheels into <dir>, where the default is the current working directory.",
        default=".",
        metavar="<dir>",
    )

    cache_parser.add_argument("cache_command", choices=["dir", "info", "list", "purge"])

    args = main_parser.parse_args(args=raw_args)

    # print("ARGS", args)
    return args



"""Support for scanning keys and key matrices

The `keypad` module provides native support to scan sets of keys or buttons,
connected independently to individual pins,
connected to a shift register,
or connected in a row-and-column matrix.

For more information about working with the `keypad` module in CircuitPython,
see `this Learn guide <https://learn.adafruit.com/key-pad-matrix-scanning-in-circuitpython>`_.
"""

from __future__ import annotations

from typing import Optional, Sequence, Tuple

import microcontroller

class Event:
    """A key transition event."""

    def __init__(
        self, key_number: int = 0, pressed: bool = True, timestamp: Optional[int] = None
    ) -> None:
        """Create a key transition event, which reports a key-pressed or key-released transition.

        :param int key_number: The key number.
        :param bool pressed: ``True`` if the key was pressed; ``False`` if it was released.
        :param int timestamp: The time in milliseconds that the keypress occurred in the `supervisor.ticks_ms` time system.  If specified as None, the current value of `supervisor.ticks_ms` is used.
        """
        ...
    key_number: int
    """The key number."""
    pressed: bool
    """``True`` if the event represents a key down (pressed) transition.
    The opposite of `released`.
    """
    released: bool
    """``True`` if the event represents a key up (released) transition.
    The opposite of `pressed`.
    """
    timestamp: int
    """The timestamp."""
    def __eq__(self, other: object) -> bool:
        """Two `Event` objects are equal if their `key_number`
        and `pressed`/`released` values are equal.
        Note that this does not compare the event timestamps.
        """
        ...
    def __hash__(self) -> int:
        """Returns a hash for the `Event`, so it can be used in dictionaries, etc..

        Note that as events with different timestamps compare equal, they also hash to the same value.
        """
        ...

class EventQueue:
    """A queue of `Event` objects, filled by a `keypad` scanner such as `Keys` or `KeyMatrix`.

    You cannot create an instance of `EventQueue` directly. Each scanner creates an
    instance when it is created.
    """

    ...
    def get(self) -> Optional[Event]:
        """Return the next key transition event. Return ``None`` if no events are pending.

        Note that the queue size is limited; see ``max_events`` in the constructor of
        a scanner such as `Keys` or `KeyMatrix`.
        If a new event arrives when the queue is full, the event is discarded, and
        `overflowed` is set to ``True``.

        :return: The next queued key transition `Event`.
        :rtype: Optional[Event]
        """
        ...
    def get_into(self, event: Event) -> bool:
        """Store the next key transition event in the supplied event, if available,
        and return ``True``.
        If there are no queued events, do not touch ``event`` and return ``False``.

        The advantage of this method over ``get()`` is that it does not allocate storage.
        Instead you can reuse an existing ``Event`` object.

        Note that the queue size is limited; see ``max_events`` in the constructor of
        a scanner such as `Keys` or `KeyMatrix`.

        :return: ``True`` if an event was available and stored, ``False`` if not.
        :rtype: bool
        """
        ...
    def clear(self) -> None:
        """Clear any queued key transition events. Also sets `overflowed` to ``False``."""
        ...
    def __bool__(self) -> bool:
        """``True`` if `len()` is greater than zero.
        This is an easy way to check if the queue is empty.
        """
        ...
    def __len__(self) -> int:
        """Return the number of events currently in the queue. Used to implement ``len()``."""
        ...
    overflowed: bool
    """``True`` if an event could not be added to the event queue because it was full. (read-only)
    Set to ``False`` by  `clear()`.
    """

class KeyMatrix:
    """Manage a 2D matrix of keys with row and column pins."""

    def __init__(
        self,
        row_pins: Sequence[microcontroller.Pin],
        column_pins: Sequence[microcontroller.Pin],
        columns_to_anodes: bool = True,
        interval: float = 0.020,
        max_events: int = 64,
    ) -> None:
        """
        Create a `Keys` object that will scan the key matrix attached to the given row and column pins.
        There should not be any external pull-ups or pull-downs on the matrix:
        ``KeyMatrix`` enables internal pull-ups or pull-downs on the pins as necessary.

        The keys are numbered sequentially from zero. A key number can be computed
        by ``row * len(column_pins) + column``.

        An `EventQueue` is created when this object is created and is available in the `events` attribute.

        :param Sequence[microcontroller.Pin] row_pins: The pins attached to the rows.
        :param Sequence[microcontroller.Pin] column_pins: The pins attached to the columns.
        :param bool columns_to_anodes: Default ``True``.
          If the matrix uses diodes, the diode anodes are typically connected to the column pins,
          and the cathodes should be connected to the row pins. If your diodes are reversed,
          set ``columns_to_anodes`` to ``False``.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        ...
    def deinit(self) -> None:
        """Stop scanning and release the pins."""
        ...
    def __enter__(self) -> KeyMatrix:
        """No-op used by Context Managers."""
        ...
    def __exit__(self) -> None:
        """Automatically deinitializes when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info."""
        ...
    def reset(self) -> None:
        """Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        ...
    key_count: int
    """The number of keys that are being scanned. (read-only)
    """
    def key_number_to_row_column(self, key_number: int) -> Tuple[int]:
        """Return the row and column for the given key number.
        The row is ``key_number // len(column_pins)``.
        The column is ``key_number % len(column_pins)``.

        :return: ``(row, column)``
        :rtype: Tuple[int]
        """
        ...
    def row_column_to_key_number(self, row: int, column: int) -> int:
        """Return the key number for a given row and column.
        The key number is ``row * len(column_pins) + column``.
        """
        ...
    events: EventQueue
    """The `EventQueue` associated with this `Keys` object. (read-only)
    """

class Keys:
    """Manage a set of independent keys."""

    def __init__(
        self,
        pins: Sequence[microcontroller.Pin],
        *,
        value_when_pressed: bool,
        pull: bool = True,
        interval: float = 0.020,
        max_events: int = 64,
    ) -> None:
        """
        Create a `Keys` object that will scan keys attached to the given sequence of pins.
        Each key is independent and attached to its own pin.

        An `EventQueue` is created when this object is created and is available in the `events` attribute.

        :param Sequence[microcontroller.Pin] pins: The pins attached to the keys.
          The key numbers correspond to indices into this sequence.
        :param bool value_when_pressed: ``True`` if the pin reads high when the key is pressed.
          ``False`` if the pin reads low (is grounded) when the key is pressed.
          All the pins must be connected in the same way.
        :param bool pull: ``True`` if an internal pull-up or pull-down should be
          enabled on each pin. A pull-up will be used if ``value_when_pressed`` is ``False``;
          a pull-down will be used if it is ``True``.
          If an external pull is already provided for all the pins, you can set ``pull`` to ``False``.
          However, enabling an internal pull when an external one is already present is not a problem;
          it simply uses slightly more current.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        ...
    def deinit(self) -> None:
        """Stop scanning and release the pins."""
        ...
    def __enter__(self) -> Keys:
        """No-op used by Context Managers."""
        ...
    def __exit__(self) -> None:
        """Automatically deinitializes when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info."""
        ...
    def reset(self) -> None:
        """Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        ...
    key_count: int
    """The number of keys that are being scanned. (read-only)
    """
    events: EventQueue
    """The `EventQueue` associated with this `Keys` object. (read-only)
    """

class ShiftRegisterKeys:
    """Manage a set of keys attached to an incoming shift register."""

    def __init__(
        self,
        *,
        clock: microcontroller.Pin,
        data: microcontroller.Pin,
        latch: microcontroller.Pin,
        value_to_latch: bool = True,
        key_count: int,
        value_when_pressed: bool,
        interval: float = 0.020,
        max_events: int = 64,
    ) -> None:
        """
        Create a `Keys` object that will scan keys attached to a parallel-in serial-out shift register
        like the 74HC165 or CD4021.
        Note that you may chain shift registers to load in as many values as you need.

        Key number 0 is the first (or more properly, the zero-th) bit read. In the
        74HC165, this bit is labeled ``Q7``. Key number 1 will be the value of ``Q6``, etc.

        An `EventQueue` is created when this object is created and is available in the `events` attribute.

        :param microcontroller.Pin clock: The shift register clock pin.
          The shift register should clock on a low-to-high transition.
        :param microcontroller.Pin data: the incoming shift register data pin
        :param microcontroller.Pin latch:
          Pin used to latch parallel data going into the shift register.
        :param bool value_to_latch: Pin state to latch data being read.
          ``True`` if the data is latched when ``latch`` goes high
          ``False`` if the data is latched when ``latch`` goes low.
          The default is ``True``, which is how the 74HC165 operates. The CD4021 latch is the opposite.
          Once the data is latched, it will be shifted out by toggling the clock pin.
        :param int key_count: number of data lines to clock in
        :param bool value_when_pressed: ``True`` if the pin reads high when the key is pressed.
          ``False`` if the pin reads low (is grounded) when the key is pressed.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        ...
    def deinit(self) -> None:
        """Stop scanning and release the pins."""
        ...
    def __enter__(self) -> Keys:
        """No-op used by Context Managers."""
        ...
    def __exit__(self) -> None:
        """Automatically deinitializes when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info."""
        ...
    def reset(self) -> None:
        """Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        ...
    key_count: int
    """The number of keys that are being scanned. (read-only)
    """
    events: EventQueue
    """The `EventQueue` associated with this `Keys` object. (read-only)
    """



import re
import sys
import subprocess
import tkinter as tk


class CheackUpdates:
    def __init__(self):
        self.os_type = sys.platform
        self.root = tk.Tk()
        self.url = f"https://pypi.org/pypi/turcar/json"

    def is_windows(self):
        return self.os_type.startswith('win')

    def is_linux(self):
        return self.os_type.startswith('linux')

    def is_mac(self):
        return self.os_type.startswith('darwin')

    def close(self):
        self.root.destroy()
    # 检查版本
    def cheack_version(self, version):
        self.root.title('新版本更新')
        self.root.resizable(False, False)
        self.root.geometry("300x200")  # 设置弹框大小
        popup_label = tk.Label(self.root, text="发现新版本，是否立即更新？", font=("Arial", 16))
        popup_label.pack(pady=20)

        popup_button_update = tk.Button(self.root, text="更新", command=lambda: self.update_bash('/home/orangepi/yb/thonnyFile/turcar.bash', version))
        popup_button_update.pack(pady=10)

        popup_button_not_update = tk.Button(self.root, text="不更新", command=self.close)
        popup_button_not_update.pack(pady=10)
        self.root.geometry(
            "500x200+{}+{}".format(int(self.root.winfo_screenwidth() / 2 - 250),
                                   int(self.root.winfo_screenheight() / 2 - 100)))  # 根据窗口大小调整位置
        self.root.mainloop()

    # 更新脚本中的版本号
    def update_bash(self, filename, new_version):
        """
        更新脚本中的版本号

        Args:
            filename: 要更新的脚本文件名
            new_version: 新的版本号，格式为 x.x.x
        """
        # 使用正则表达式匹配并替换版本号
        with open(filename, 'r') as file:
            content = file.read()
            new_content = re.sub(r'(VERSION=)\d+\.\d+\.\d+', r'\g<1>' + new_version, content)

        # 将更新后的内容写回脚本文件
        with open(filename, 'w') as file:
            file.write(new_content)

        # 指定版本更新
        execute_bash_command(filename)
        self.root.destroy()


# 执行 Bash 命令
def execute_bash_command(command):
    # subprocess.run(command, shell=True)
    # 执行命令并获取输出
    result = subprocess.run(command, shell=True, capture_output=True, text=True)

    # 输出命令执行结果
    print("命令输出：")
    print(result.stdout)
    print()

    # 输出命令的退出状态码
    print("退出状态码：", result.returncode)



class Font:
    """
    Object that represents a font for writing text.

    The font object will be a font that is the "best" match based on the parameters given and available fonts installed.

    Class Attributes:
        DEFAULT (Font): The default font.

    Attributes:
        family (str): The family of the font.
        style (str): A string describing the font style.
        width (int): The width of the widest character of the set.
        height (int): The height of the font.

    Args:
        family (str): The preferred font family or None to use the default value.
        size (int): The preferred font size. Most fonts have sizes between 6 and 24. This is the “point” size and not the same as height.
        bold (bool): When True, prefer bold fonts.
        monospace (bool): When True prefer monospaced fonts. This is useful for aligning multiple rows of text.
        lang (str): A language code, such as 'en' or 'zh-cn' or None to use the default language. [1]
        script (str): A unicode script identifier such as 'Runr' or None.
    """
    DEFAULT = None  # type: Font

    def __init__(self, family: str = None, size: int = 12, bold: bool = False, monospace: bool = False, lang: str = None, script: str = None):
        self.family = ''  # type: str
        self.style = ''  # type: str
        self.width = 0  # type: int
        self.height = 0  # type: int

    def text_width(self, text: str) -> int:
        """
        Gets the width of the text when the text is drawn using this font.

        Returns:
            The width in pixels.
        """

        return 0

    def text_height(self, text: str) -> int:
        """
        Gets the height of the text when the text is drawn using this font.

        Returns:
            The height in pixels.
        """

        return 0

Font.DEFAULT = Font('Lucida', 12)


"""Support for edge counting

The `countio` module contains logic to read and count edge transitions

For more information on the applications of counting edges, see
`this Learn Guide on sequential circuits
<https://learn.adafruit.com/digital-circuits-4-sequential-circuits>`_.

All classes change hardware state and should be deinitialized when they
are no longer needed if the program continues after use. To do so, either
call :py:meth:`!deinit` or use a context manager. See
:ref:`lifetime-and-contextmanagers` for more info."""

from __future__ import annotations

from typing import Optional

import digitalio
import microcontroller

class Edge:
    """Enumerates which signal transitions can be counted."""

    def __init__(self) -> None:
        """Enum-like class to define which signal transitions to count."""
        ...
    RISE: Edge
    """Count the rising edges."""

    FALL: Edge
    """Count the falling edges."""

    RISE_AND_FALL: Edge
    """Count the rising and falling edges.

    **Limitations:** ``RISE_AND_FALL`` is not available to RP2040 due to hardware limitations.
    """

class Counter:
    """Count the number of rising- and/or falling-edge transitions on a given pin."""

    def __init__(
        self,
        pin: microcontroller.Pin,
        *,
        edge: Edge = Edge.FALL,
        pull: Optional[digitalio.Pull] = None,
    ) -> None:
        """Create a Counter object associated with the given pin that counts
        rising- and/or falling-edge transitions. At least one of ``rise`` and ``fall`` must be True.
        The default is to count only falling edges, and is for historical backward compatibility.

        :param ~microcontroller.Pin pin: pin to monitor
        :param Edge edge: which edge transitions to count
        :param Optional[digitalio.Pull] pull: enable a pull-up or pull-down if not None


        For example::

            import board
            import countio

            # Count rising edges only.
            pin_counter = countio.Counter(board.D1, edge=countio.Edge.RISE)
            # Reset the count after 100 counts.
            while True:
                if pin_counter.count >= 100:
                    pin_counter.reset()
                print(pin_counter.count)
        """
        ...
    def deinit(self) -> None:
        """Deinitializes the Counter and releases any hardware resources for reuse."""
    def __enter__(self) -> Counter:
        """No-op used by Context Managers."""
    def __exit__(self) -> None:
        """Automatically deinitializes the hardware when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info."""
    count: int
    """The current count in terms of pulses."""
    def reset(self) -> None:
        """Resets the count back to 0."""



"""Traceback Module

This module provides a standard interface to print stack traces of programs.
This is useful when you want to print stack traces under program control.

|see_cpython_module| :mod:`cpython:traceback`.
"""

from __future__ import annotations

import io
from types import TracebackType
from typing import List, Optional, Type

...

def format_exception(
    exc: BaseException | Type[BaseException],
    /,
    value: Optional[BaseException] = None,
    tb: Optional[TracebackType] = None,
    limit: Optional[int] = None,
    chain: Optional[bool] = True,
) -> List[str]:
    """Format a stack trace and the exception information.

    If the exception value is passed in ``exc``, then this exception value and its
    associated traceback are used. This is compatible with CPython 3.10 and newer.

    If the exception value is passed in ``value``, then any value passed in for
    ``exc`` is ignored.  ``value`` is used as the exception value and the
    traceback in the ``tb`` argument is used.  In this case, if ``tb`` is None,
    no traceback will be shown. This is compatible with CPython 3.5 and
    newer.

    The arguments have the same meaning as the corresponding arguments
    to print_exception().  The return value is a list of strings, each
    ending in a newline and some containing internal newlines.  When
    these lines are concatenated and printed, exactly the same text is
    printed as does print_exception().

    :param exc: The exception. Must be an instance of `BaseException`. Unused if value is specified.
    :param value: If specified, is used in place of ``exc``.
    :param TracebackType tb: When value is alsp specified, ``tb`` is used in place of the exception's own traceback. If `None`, the traceback will not be printed.
    :param int limit: Print up to limit stack trace entries (starting from the caller’s frame) if limit is positive.
                      Otherwise, print the last ``abs(limit)`` entries. If limit is omitted or None, all entries are printed.
    :param bool chain: If `True` then chained exceptions will be printed.
    """

def print_exception(
    exc: BaseException | Type[BaseException],
    /,
    value: Optional[BaseException] = None,
    tb: Optional[TracebackType] = None,
    limit: Optional[int] = None,
    file: Optional[io.FileIO] = None,
    chain: Optional[bool] = True,
) -> None:
    """Prints exception information and stack trace entries.

    If the exception value is passed in ``exc``, then this exception value and its
    associated traceback are used. This is compatible with CPython 3.10 and newer.

    If the exception value is passed in ``value``, then any value passed in for
    ``exc`` is ignored.  ``value`` is used as the exception value and the
    traceback in the ``tb`` argument is used.  In this case, if ``tb`` is None,
    no traceback will be shown. This is compatible with CPython 3.5 and
    newer.

    :param exc: The exception. Must be an instance of `BaseException`. Unused if value is specified.
    :param value: If specified, is used in place of ``exc``.
    :param tb: When value is alsp specified, ``tb`` is used in place of the exception's own traceback. If `None`, the traceback will not be printed.
    :param int limit: Print up to limit stack trace entries (starting from the caller’s frame) if limit is positive.
                      Otherwise, print the last ``abs(limit)`` entries. If limit is omitted or None, all entries are printed.
    :param io.FileIO file: If file is omitted or `None`, the output goes to `sys.stderr`; otherwise it should be an open
                           file or file-like object to receive the output.
    :param bool chain: If `True` then chained exceptions will be printed.

    """
    ...



"""Collection of bitmap manipulation tools

.. note:: If you're looking for information about displaying bitmaps on
    screens in CircuitPython, see `this Learn guide
    <https://learn.adafruit.com/circuitpython-display-support-using-displayio>`_
    for information about using the :py:mod:`displayio` module.
"""

from __future__ import annotations

import typing
from typing import Optional, Tuple

import displayio
from circuitpython_typing import ReadableBuffer

def rotozoom(
    dest_bitmap: displayio.Bitmap,
    source_bitmap: displayio.Bitmap,
    *,
    ox: int,
    oy: int,
    dest_clip0: Tuple[int, int],
    dest_clip1: Tuple[int, int],
    px: int,
    py: int,
    source_clip0: Tuple[int, int],
    source_clip1: Tuple[int, int],
    angle: float,
    scale: float,
    skip_index: int,
) -> None:
    """Inserts the source bitmap region into the destination bitmap with rotation
    (angle), scale and clipping (both on source and destination bitmaps).

    :param bitmap dest_bitmap: Destination bitmap that will be copied into
    :param bitmap source_bitmap: Source bitmap that contains the graphical region to be copied
    :param int ox: Horizontal pixel location in destination bitmap where source bitmap
           point (px,py) is placed. Defaults to None which causes it to use the horizontal
           midway point of the destination bitmap.
    :param int oy: Vertical pixel location in destination bitmap where source bitmap
           point (px,py) is placed. Defaults to None which causes it to use the vertical
           midway point of the destination bitmap.
    :param Tuple[int,int] dest_clip0: First corner of rectangular destination clipping
           region that constrains region of writing into destination bitmap
    :param Tuple[int,int] dest_clip1: Second corner of rectangular destination clipping
           region that constrains region of writing into destination bitmap
    :param int px: Horizontal pixel location in source bitmap that is placed into the
           destination bitmap at (ox,oy). Defaults to None which causes it to use the
           horizontal midway point in the source bitmap.
    :param int py: Vertical pixel location in source bitmap that is placed into the
           destination bitmap at (ox,oy). Defaults to None which causes it to use the
           vertical midway point in the source bitmap.
    :param Tuple[int,int] source_clip0: First corner of rectangular source clipping
           region that constrains region of reading from the source bitmap
    :param Tuple[int,int] source_clip1: Second corner of rectangular source clipping
           region that constrains region of reading from the source bitmap
    :param float angle: Angle of rotation, in radians (positive is clockwise direction).
           Defaults to None which gets treated as 0.0 radians or no rotation.
    :param float scale: Scaling factor. Defaults to None which gets treated as 1.0 or same
           as original source size.
    :param int skip_index: Bitmap palette index in the source that will not be copied,
           set to None to copy all pixels"""
    ...

def alphablend(
    dest_bitmap: displayio.Bitmap,
    source_bitmap_1: displayio.Bitmap,
    source_bitmap_2: displayio.Bitmap,
    colorspace: displayio.Colorspace,
    factor1: float = 0.5,
    factor2: Optional[float] = None,
) -> None:
    """Alpha blend the two source bitmaps into the destination.

    It is permitted for the destination bitmap to be one of the two
    source bitmaps.

    :param bitmap dest_bitmap: Destination bitmap that will be written into
    :param bitmap source_bitmap_1: The first source bitmap
    :param bitmap source_bitmap_2: The second source bitmap
    :param float factor1: The proportion of bitmap 1 to mix in
    :param float factor2: The proportion of bitmap 2 to mix in.  If specified as `None`, ``1-factor1`` is used.  Usually the proportions should sum to 1.
    :param displayio.Colorspace colorspace: The colorspace of the bitmaps. They must all have the same colorspace.  Only the following colorspaces are permitted:  ``L8``, ``RGB565``, ``RGB565_SWAPPED``, ``BGR565`` and ``BGR565_SWAPPED``.

    For the L8 colorspace, the bitmaps must have a bits-per-value of 8.
    For the RGB colorspaces, they must have a bits-per-value of 16."""

def fill_region(
    dest_bitmap: displayio.Bitmap, x1: int, y1: int, x2: int, y2: int, value: int
) -> None:
    """Draws the color value into the destination bitmap within the
    rectangular region bounded by (x1,y1) and (x2,y2), exclusive.

    :param bitmap dest_bitmap: Destination bitmap that will be written into
    :param int x1: x-pixel position of the first corner of the rectangular fill region
    :param int y1: y-pixel position of the first corner of the rectangular fill region
    :param int x2: x-pixel position of the second corner of the rectangular fill region (exclusive)
    :param int y2: y-pixel position of the second corner of the rectangular fill region (exclusive)
    :param int value: Bitmap palette index that will be written into the rectangular
           fill region in the destination bitmap"""
    ...

def boundary_fill(
    dest_bitmap: displayio.Bitmap,
    x: int,
    y: int,
    fill_color_value: int,
    replaced_color_value: int,
) -> None:
    """Draws the color value into the destination bitmap enclosed
    area of pixels of the background_value color. Like "Paint Bucket"
    fill tool.

    :param bitmap dest_bitmap: Destination bitmap that will be written into
    :param int x: x-pixel position of the first pixel to check and fill if needed
    :param int y: y-pixel position of the first pixel to check and fill if needed
    :param int fill_color_value: Bitmap palette index that will be written into the
           enclosed area in the destination bitmap
    :param int replaced_color_value: Bitmap palette index that will filled with the
           value color in the enclosed area in the destination bitmap"""
    ...

def draw_line(
    dest_bitmap: displayio.Bitmap, x1: int, y1: int, x2: int, y2: int, value: int
) -> None:
    """Draws a line into a bitmap specified two endpoints (x1,y1) and (x2,y2).

    :param bitmap dest_bitmap: Destination bitmap that will be written into
    :param int x1: x-pixel position of the line's first endpoint
    :param int y1: y-pixel position of the line's first endpoint
    :param int x2: x-pixel position of the line's second endpoint
    :param int y2: y-pixel position of the line's second endpoint
    :param int value: Bitmap palette index that will be written into the
           line in the destination bitmap"""
    ...

def draw_polygon(
    dest_bitmap: displayio.Bitmap,
    xs: ReadableBuffer,
    ys: ReadableBuffer,
    value: int,
    close: Optional[bool] = True,
) -> None:
    """Draw a polygon connecting points on provided bitmap with provided value

    :param bitmap dest_bitmap: Destination bitmap that will be written into
    :param ReadableBuffer xs: x-pixel position of the polygon's vertices
    :param ReadableBuffer ys: y-pixel position of the polygon's vertices
    :param int value: Bitmap palette index that will be written into the
           line in the destination bitmap
    :param bool close: (Optional) Whether to connect first and last point. (True)

    .. code-block:: Python

       import board
       import displayio
       import bitmaptools

       display = board.DISPLAY
       main_group = displayio.Group()
       display.root_group = main_group

       palette = displayio.Palette(3)
       palette[0] = 0xffffff
       palette[1] = 0x0000ff
       palette[2] = 0xff0000

       bmp = displayio.Bitmap(128,128, 3)
       bmp.fill(0)

       xs = bytes([4, 101, 101, 19])
       ys = bytes([4, 19,  121, 101])
       bitmaptools.draw_polygon(bmp, xs, ys, 1)

       xs = bytes([14, 60, 110])
       ys = bytes([14, 24,  90])
       bitmaptools.draw_polygon(bmp, xs, ys, 2)

       tilegrid = displayio.TileGrid(bitmap=bmp, pixel_shader=palette)
       main_group.append(tilegrid)

       while True:
           pass
    """
    ...

def arrayblit(
    bitmap: displayio.Bitmap,
    data: ReadableBuffer,
    x1: int = 0,
    y1: int = 0,
    x2: Optional[int] = None,
    y2: Optional[int] = None,
    skip_index: Optional[int] = None,
) -> None:
    """Inserts pixels from ``data`` into the rectangle of width×height pixels with the upper left corner at ``(x,y)``

    The values from ``data`` are taken modulo the number of color values
    available in the destination bitmap.

    If x1 or y1 are not specified, they are taken as 0.  If x2 or y2
    are not specified, or are given as -1, they are taken as the width
    and height of the image.

    The coordinates affected by the blit are ``x1 <= x < x2`` and ``y1 <= y < y2``.

    ``data`` must contain at least as many elements as required.  If it
    contains excess elements, they are ignored.

    The blit takes place by rows, so the first elements of ``data`` go
    to the first row, the next elements to the next row, and so on.

    :param displayio.Bitmap bitmap: A writable bitmap
    :param ReadableBuffer data: Buffer containing the source pixel values
    :param int x1: The left corner of the area to blit into (inclusive)
    :param int y1: The top corner of the area to blit into (inclusive)
    :param int x2: The right of the area to blit into (exclusive)
    :param int y2: The bottom corner of the area to blit into (exclusive)
    :param int skip_index: Bitmap palette index in the source that will not be copied,
            set to None to copy all pixels
    """
    ...

def readinto(
    bitmap: displayio.Bitmap,
    file: typing.BinaryIO,
    bits_per_pixel: int,
    element_size: int = 1,
    reverse_pixels_in_element: bool = False,
    swap_bytes_in_element: bool = False,
    reverse_rows: bool = False,
) -> None:
    """Reads from a binary file into a bitmap.

    The file must be positioned so that it consists of ``bitmap.height`` rows of pixel data, where each row is the smallest multiple of ``element_size`` bytes that can hold ``bitmap.width`` pixels.

    The bytes in an element can be optionally swapped, and the pixels in an element can be reversed.  Also, the
    row loading direction can be reversed, which may be requires for loading certain bitmap files.

    This function doesn't parse image headers, but is useful to speed up loading of uncompressed image formats such as PCF glyph data.

    :param displayio.Bitmap bitmap: A writable bitmap
    :param typing.BinaryIO file: A file opened in binary mode
    :param int bits_per_pixel: Number of bits per pixel.  Values 1, 2, 4, 8, 16, 24, and 32 are supported;
    :param int element_size: Number of bytes per element.  Values of 1, 2, and 4 are supported, except that 24 ``bits_per_pixel`` requires 1 byte per element.
    :param bool reverse_pixels_in_element: If set, the first pixel in a word is taken from the Most Significant Bits; otherwise, it is taken from the Least Significant Bits.
    :param bool swap_bytes_in_element: If the ``element_size`` is not 1, then reverse the byte order of each element read.
    :param bool reverse_rows: Reverse the direction of the row loading (required for some bitmap images).
    """
    ...

class DitherAlgorithm:
    """Identifies the algorithm for dither to use"""

    Atkinson: "DitherAlgorithm"
    """The classic Atkinson dither, often associated with the Hypercard esthetic"""

    FloydStenberg: "DitherAlgorithm"
    """The Floyd-Stenberg dither"""

def dither(
    dest_bitmap: displayio.Bitmap,
    source_bitmapp: displayio.Bitmap,
    source_colorspace: displayio.Colorspace,
    algorithm: DitherAlgorithm = DitherAlgorithm.Atkinson,
) -> None:
    """Convert the input image into a 2-level output image using the given dither algorithm.

    :param bitmap dest_bitmap: Destination bitmap.  It must have a value_count of 2 or 65536.  The stored values are 0 and the maximum pixel value.
    :param bitmap source_bitmap: Source bitmap that contains the graphical region to be dithered.  It must have a value_count of 65536.
    :param colorspace: The colorspace of the image.  The supported colorspaces are ``RGB565``, ``BGR565``, ``RGB565_SWAPPED``, and ``BGR565_SWAPPED``
    :param algorithm: The dither algorithm to use, one of the `DitherAlgorithm` values.
    """
    ...



__all__ = ["Font", "Image", "ImageFile", "SoundFile"]

from .__stub.__imagefile import ImageFile
from .__stub.__soundfile import SoundFile
from .__stub.__font import Font
from .__stub.__image import Image


"""
functions related to the hardware.

Descriptions taken from:
https://raw.githubusercontent.com/micropython/micropython/master/docs/library/machine.rst.
====================================================

.. module:: machine
   :synopsis: functions related to the hardware

   

   The ``machine`` module contains specific functions related to the hardware
   on a particular board. Most functions in this module allow to achieve direct
   and unrestricted access to and control of hardware blocks on a system
   (like CPU, timers, buses, etc.). Used incorrectly, this can lead to
   malfunction, lockups, crashes of your board, and in extreme cases, hardware
   damage.
   
   .. _machine_callbacks:
   
   A note of callbacks used by functions and class methods of :mod:`machine` module:
   all these callbacks should be considered as executing in an interrupt context.
   This is true for both physical devices with IDs >= 0 and "virtual" devices
   with negative IDs like -1 (these "virtual" devices are still thin shims on
   top of real hardware and real hardware interrupts). See :ref:`isr_rules`.
"""

__author__ = "Howard C Lovatt"
__copyright__ = "Howard C Lovatt, 2020 onwards."
__license__ = "MIT https://opensource.org/licenses/MIT (as used by MicroPython)."
__version__ = "7.3.9"  # Version set by https://github.com/hlovatt/tag2ver

from typing import overload, NoReturn, Callable
from typing import Sequence, ClassVar, Any, Final

from uos import AbstractBlockDev
from uio import AnyReadableBuf, AnyWritableBuf

def reset() -> NoReturn:
    """
   Resets the device in a manner similar to pushing the external RESET
   button.
   """

def soft_reset() -> NoReturn:
    """
   Performs a soft reset of the interpreter, deleting all Python objects and
   resetting the Python heap.  It tries to retain the method by which the user
   is connected to the MicroPython REPL (eg serial, USB, Wifi).
   """

def reset_cause() -> int:
    """
   Get the reset cause. See :ref:`constants <machine_constants>` for the possible return values.
   """

def disable_irq() -> bool:
    """
   Disable interrupt requests.
   Returns the previous IRQ state which should be considered an opaque value.
   This return value should be passed to the `enable_irq()` function to restore
   interrupts to their original state, before `disable_irq()` was called.
   """

def enable_irq(state: bool = True, /) -> None:
    """
   Re-enable interrupt requests.
   The *state* parameter should be the value that was returned from the most
   recent call to the `disable_irq()` function.
   """

@overload
def freq() -> int:
    """
    Returns the CPU frequency in hertz.
    
    On some ports this can also be used to set the CPU frequency by passing in *hz*.
   """

@overload
def freq(hz: int, /) -> None:
    """
    Returns the CPU frequency in hertz.
    
    On some ports this can also be used to set the CPU frequency by passing in *hz*.
   """

def idle() -> None:
    """
   Gates the clock to the CPU, useful to reduce power consumption at any time during
   short or long periods. Peripherals continue working and execution resumes as soon
   as any interrupt is triggered (on many ports this includes system timer
   interrupt occurring at regular intervals on the order of millisecond).
   """

def sleep() -> None:
    """
   .. note:: This function is deprecated, use `lightsleep()` instead with no arguments.
   """

@overload
def lightsleep() -> None:
    """
   Stops execution in an attempt to enter a low power state.
   
   If *time_ms* is specified then this will be the maximum time in milliseconds that
   the sleep will last for.  Otherwise the sleep can last indefinitely.
   
   With or without a timeout, execution may resume at any time if there are events
   that require processing.  Such events, or wake sources, should be configured before
   sleeping, like `Pin` change or `RTC` timeout.
   
   The precise behaviour and power-saving capabilities of lightsleep and deepsleep is
   highly dependent on the underlying hardware, but the general properties are:
   
   * A lightsleep has full RAM and state retention.  Upon wake execution is resumed
     from the point where the sleep was requested, with all subsystems operational.
   
   * A deepsleep may not retain RAM or any other state of the system (for example
     peripherals or network interfaces).  Upon wake execution is resumed from the main
     script, similar to a hard or power-on reset. The `reset_cause()` function will
     return `machine.DEEPSLEEP` and this can be used to distinguish a deepsleep wake
     from other resets.
   """

@overload
def lightsleep(time_ms: int, /) -> None:
    """
   Stops execution in an attempt to enter a low power state.
   
   If *time_ms* is specified then this will be the maximum time in milliseconds that
   the sleep will last for.  Otherwise the sleep can last indefinitely.
   
   With or without a timeout, execution may resume at any time if there are events
   that require processing.  Such events, or wake sources, should be configured before
   sleeping, like `Pin` change or `RTC` timeout.
   
   The precise behaviour and power-saving capabilities of lightsleep and deepsleep is
   highly dependent on the underlying hardware, but the general properties are:
   
   * A lightsleep has full RAM and state retention.  Upon wake execution is resumed
     from the point where the sleep was requested, with all subsystems operational.
   
   * A deepsleep may not retain RAM or any other state of the system (for example
     peripherals or network interfaces).  Upon wake execution is resumed from the main
     script, similar to a hard or power-on reset. The `reset_cause()` function will
     return `machine.DEEPSLEEP` and this can be used to distinguish a deepsleep wake
     from other resets.
   """

@overload
def deepsleep() -> NoReturn:
    """
   Stops execution in an attempt to enter a low power state.
   
   If *time_ms* is specified then this will be the maximum time in milliseconds that
   the sleep will last for.  Otherwise the sleep can last indefinitely.
   
   With or without a timeout, execution may resume at any time if there are events
   that require processing.  Such events, or wake sources, should be configured before
   sleeping, like `Pin` change or `RTC` timeout.
   
   The precise behaviour and power-saving capabilities of lightsleep and deepsleep is
   highly dependent on the underlying hardware, but the general properties are:
   
   * A lightsleep has full RAM and state retention.  Upon wake execution is resumed
     from the point where the sleep was requested, with all subsystems operational.
   
   * A deepsleep may not retain RAM or any other state of the system (for example
     peripherals or network interfaces).  Upon wake execution is resumed from the main
     script, similar to a hard or power-on reset. The `reset_cause()` function will
     return `machine.DEEPSLEEP` and this can be used to distinguish a deepsleep wake
     from other resets.
   """

@overload
def deepsleep(time_ms: int, /) -> NoReturn:
    """
   Stops execution in an attempt to enter a low power state.
   
   If *time_ms* is specified then this will be the maximum time in milliseconds that
   the sleep will last for.  Otherwise the sleep can last indefinitely.
   
   With or without a timeout, execution may resume at any time if there are events
   that require processing.  Such events, or wake sources, should be configured before
   sleeping, like `Pin` change or `RTC` timeout.
   
   The precise behaviour and power-saving capabilities of lightsleep and deepsleep is
   highly dependent on the underlying hardware, but the general properties are:
   
   * A lightsleep has full RAM and state retention.  Upon wake execution is resumed
     from the point where the sleep was requested, with all subsystems operational.
   
   * A deepsleep may not retain RAM or any other state of the system (for example
     peripherals or network interfaces).  Upon wake execution is resumed from the main
     script, similar to a hard or power-on reset. The `reset_cause()` function will
     return `machine.DEEPSLEEP` and this can be used to distinguish a deepsleep wake
     from other resets.
   """

def wake_reason() -> int:
    """
   Get the wake reason. See :ref:`constants <machine_constants>` for the possible return values.
   
   Availability: ESP32, WiPy.
   """

def unique_id() -> bytes:
    """
   Returns a byte string with a unique identifier of a board/SoC. It will vary
   from a board/SoC instance to another, if underlying hardware allows. Length
   varies by hardware (so use substring of a full value if you expect a short
   ID). In some MicroPython ports, ID corresponds to the network MAC address.
   """

def time_pulse_us(pin: Pin, pulse_level: int, timeout_us: int = 1_000_000, /) -> int:
    """
   Time a pulse on the given *pin*, and return the duration of the pulse in
   microseconds.  The *pulse_level* argument should be 0 to time a low pulse
   or 1 to time a high pulse.
   
   If the current input value of the pin is different to *pulse_level*,
   the function first (*) waits until the pin input becomes equal to *pulse_level*,
   then (**) times the duration that the pin is equal to *pulse_level*.
   If the pin is already equal to *pulse_level* then timing starts straight away.
   
   The function will return -2 if there was timeout waiting for condition marked
   (*) above, and -1 if there was timeout during the main measurement, marked (**)
   above. The timeout is the same for both cases and given by *timeout_us* (which
   is in microseconds).
   """

def rng() -> int:
    """
   Return a 24-bit software generated random number.
   
   Availability: WiPy.
   """

IDLE: Final[int] = ...
"""
IRQ wake values.
"""

SLEEP: Final[int] = ...
"""
IRQ wake values.
"""

DEEPSLEEP: Final[int] = ...
"""
IRQ wake values.
"""

PWRON_RESET: Final[int] = ...
"""
Reset causes.
"""

HARD_RESET: Final[int] = ...
"""
Reset causes.
"""

WDT_RESET: Final[int] = ...
"""
Reset causes.
"""

DEEPSLEEP_RESET: Final[int] = ...
"""
Reset causes.
"""

SOFT_RESET: Final[int] = ...
"""
Reset causes.
"""

WLAN_WAKE: Final[int] = ...
"""
Wake-up reasons.
"""

PIN_WAKE: Final[int] = ...
"""
Wake-up reasons.
"""

RTC_WAKE: Final[int] = ...
"""
Wake-up reasons.
"""

class Pin:
    """
   A pin object is used to control I/O pins (also known as GPIO - general-purpose
   input/output).  Pin objects are commonly associated with a physical pin that can
   drive an output voltage and read input voltages.  The pin class has methods to set the mode of
   the pin (IN, OUT, etc) and methods to get and set the digital logic level.
   For analog control of a pin, see the :class:`ADC` class.
   
   A pin object is constructed by using an identifier which unambiguously
   specifies a certain I/O pin.  The allowed forms of the identifier and the
   physical pin that the identifier maps to are port-specific.  Possibilities
   for the identifier are an integer, a string or a tuple with port and pin
   number.
   
   Usage Model::
   
       from machine import Pin
   
       # create an output pin on pin #0
       p0 = Pin(0, Pin.OUT)
   
       # set the value low then high
       p0.value(0)
       p0.value(1)
   
       # create an input pin on pin #2, with a pull up resistor
       p2 = Pin(2, Pin.IN, Pin.PULL_UP)
   
       # read and print the pin value
       print(p2.value())
   
       # reconfigure pin #0 in input mode with a pull down resistor
       p0.init(p0.IN, p0.PULL_DOWN)
   
       # configure an irq callback
       p0.irq(lambda p:print(p))
   """

    IN: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    OUT: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    OPEN_DRAIN: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    ALT: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    ALT_OPEN_DRAIN: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    ANALOG: ClassVar[int] = ...
    """
Selects the pin mode.
   """

    PULL_UP: ClassVar[int] = ...
    """
Selects whether there is a pull up/down resistor.  Use the value
   ``None`` for no pull.
   """

    PULL_DOWN: ClassVar[int] = ...
    """
Selects whether there is a pull up/down resistor.  Use the value
   ``None`` for no pull.
   """

    PULL_HOLD: ClassVar[int] = ...
    """
Selects whether there is a pull up/down resistor.  Use the value
   ``None`` for no pull.
   """

    LOW_POWER: ClassVar[int] = ...
    """
Selects the pin drive strength.
   """

    MED_POWER: ClassVar[int] = ...
    """
Selects the pin drive strength.
   """

    HIGH_POWER: ClassVar[int] = ...
    """
Selects the pin drive strength.
   """

    IRQ_FALLING: ClassVar[int] = ...
    """
Selects the IRQ trigger type.
   """

    IRQ_RISING: ClassVar[int] = ...
    """
Selects the IRQ trigger type.
   """

    IRQ_LOW_LEVEL: ClassVar[int] = ...
    """
Selects the IRQ trigger type.
   """

    IRQ_HIGH_LEVEL: ClassVar[int] = ...
    """
Selects the IRQ trigger type.
   """
    def __init__(
        self,
        id: Any,
        /,
        mode: int = -1,
        pull: int = -1,
        *,
        value: Any = None,
        drive: int | None = None,
        alt: int | None = None,
    ):
        """
      Access the pin peripheral (GPIO pin) associated with the given ``id``.  If
      additional arguments are given in the constructor then they are used to initialise
      the pin.  Any settings that are not specified will remain in their previous state.
      
      The arguments are:
      
        - ``id`` is mandatory and can be an arbitrary object.  Among possible value
          types are: int (an internal Pin identifier), str (a Pin name), and tuple
          (pair of [port, pin]).
      
        - ``mode`` specifies the pin mode, which can be one of:
      
          - ``Pin.IN`` - Pin is configured for input.  If viewed as an output the pin
            is in high-impedance state.
      
          - ``Pin.OUT`` - Pin is configured for (normal) output.
      
          - ``Pin.OPEN_DRAIN`` - Pin is configured for open-drain output. Open-drain
            output works in the following way: if the output value is set to 0 the pin
            is active at a low level; if the output value is 1 the pin is in a high-impedance
            state.  Not all ports implement this mode, or some might only on certain pins.
      
          - ``Pin.ALT`` - Pin is configured to perform an alternative function, which is
            port specific.  For a pin configured in such a way any other Pin methods
            (except :meth:`Pin.init`) are not applicable (calling them will lead to undefined,
            or a hardware-specific, result).  Not all ports implement this mode.
      
          - ``Pin.ALT_OPEN_DRAIN`` - The Same as ``Pin.ALT``, but the pin is configured as
            open-drain.  Not all ports implement this mode.
      
          - ``Pin.ANALOG`` - Pin is configured for analog input, see the :class:`ADC` class.
      
        - ``pull`` specifies if the pin has a (weak) pull resistor attached, and can be
          one of:
      
          - ``None`` - No pull up or down resistor.
          - ``Pin.PULL_UP`` - Pull up resistor enabled.
          - ``Pin.PULL_DOWN`` - Pull down resistor enabled.
      
        - ``value`` is valid only for Pin.OUT and Pin.OPEN_DRAIN modes and specifies initial
          output pin value if given, otherwise the state of the pin peripheral remains
          unchanged.
      
        - ``drive`` specifies the output power of the pin and can be one of: ``Pin.LOW_POWER``,
          ``Pin.MED_POWER`` or ``Pin.HIGH_POWER``.  The actual current driving capabilities
          are port dependent.  Not all ports implement this argument.
      
        - ``alt`` specifies an alternate function for the pin and the values it can take are
          port dependent.  This argument is valid only for ``Pin.ALT`` and ``Pin.ALT_OPEN_DRAIN``
          modes.  It may be used when a pin supports more than one alternate function.  If only
          one pin alternate function is supported the this argument is not required.  Not all
          ports implement this argument.
      
      As specified above, the Pin class allows to set an alternate function for a particular
      pin, but it does not specify any further operations on such a pin.  Pins configured in
      alternate-function mode are usually not used as GPIO but are instead driven by other
      hardware peripherals.  The only operation supported on such a pin is re-initialising,
      by calling the constructor or :meth:`Pin.init` method.  If a pin that is configured in
      alternate-function mode is re-initialised with ``Pin.IN``, ``Pin.OUT``, or
      ``Pin.OPEN_DRAIN``, the alternate function will be removed from the pin.
      """
    def init(
        self,
        mode: int = -1,
        pull: int = -1,
        *,
        value: Any = None,
        drive: int | None = None,
        alt: int | None = None,
    ) -> None:
        """
      Re-initialise the pin using the given parameters.  Only those arguments that
      are specified will be set.  The rest of the pin peripheral state will remain
      unchanged.  See the constructor documentation for details of the arguments.
      
      Returns ``None``.
      """
    @overload
    def value(self) -> int:
        """
      This method allows to set and get the value of the pin, depending on whether
      the argument ``x`` is supplied or not.
      
      If the argument is omitted then this method gets the digital logic level of
      the pin, returning 0 or 1 corresponding to low and high voltage signals
      respectively.  The behaviour of this method depends on the mode of the pin:
      
        - ``Pin.IN`` - The method returns the actual input value currently present
          on the pin.
        - ``Pin.OUT`` - The behaviour and return value of the method is undefined.
        - ``Pin.OPEN_DRAIN`` - If the pin is in state '0' then the behaviour and
          return value of the method is undefined.  Otherwise, if the pin is in
          state '1', the method returns the actual input value currently present
          on the pin.
      
      If the argument is supplied then this method sets the digital logic level of
      the pin.  The argument ``x`` can be anything that converts to a boolean.
      If it converts to ``True``, the pin is set to state '1', otherwise it is set
      to state '0'.  The behaviour of this method depends on the mode of the pin:
      
        - ``Pin.IN`` - The value is stored in the output buffer for the pin.  The
          pin state does not change, it remains in the high-impedance state.  The
          stored value will become active on the pin as soon as it is changed to
          ``Pin.OUT`` or ``Pin.OPEN_DRAIN`` mode.
        - ``Pin.OUT`` - The output buffer is set to the given value immediately.
        - ``Pin.OPEN_DRAIN`` - If the value is '0' the pin is set to a low voltage
          state.  Otherwise the pin is set to high-impedance state.
      
      When setting the value this method returns ``None``.
      """
    @overload
    def value(self, x: Any, /) -> None:
        """
      This method allows to set and get the value of the pin, depending on whether
      the argument ``x`` is supplied or not.
      
      If the argument is omitted then this method gets the digital logic level of
      the pin, returning 0 or 1 corresponding to low and high voltage signals
      respectively.  The behaviour of this method depends on the mode of the pin:
      
        - ``Pin.IN`` - The method returns the actual input value currently present
          on the pin.
        - ``Pin.OUT`` - The behaviour and return value of the method is undefined.
        - ``Pin.OPEN_DRAIN`` - If the pin is in state '0' then the behaviour and
          return value of the method is undefined.  Otherwise, if the pin is in
          state '1', the method returns the actual input value currently present
          on the pin.
      
      If the argument is supplied then this method sets the digital logic level of
      the pin.  The argument ``x`` can be anything that converts to a boolean.
      If it converts to ``True``, the pin is set to state '1', otherwise it is set
      to state '0'.  The behaviour of this method depends on the mode of the pin:
      
        - ``Pin.IN`` - The value is stored in the output buffer for the pin.  The
          pin state does not change, it remains in the high-impedance state.  The
          stored value will become active on the pin as soon as it is changed to
          ``Pin.OUT`` or ``Pin.OPEN_DRAIN`` mode.
        - ``Pin.OUT`` - The output buffer is set to the given value immediately.
        - ``Pin.OPEN_DRAIN`` - If the value is '0' the pin is set to a low voltage
          state.  Otherwise the pin is set to high-impedance state.
      
      When setting the value this method returns ``None``.
      """
    @overload
    def __call__(self) -> int:
        """
      Pin objects are callable.  The call method provides a (fast) shortcut to set
      and get the value of the pin.  It is equivalent to Pin.value([x]).
      See :meth:`Pin.value` for more details.
      """
    @overload
    def __call__(self, x: Any, /) -> None:
        """
      Pin objects are callable.  The call method provides a (fast) shortcut to set
      and get the value of the pin.  It is equivalent to Pin.value([x]).
      See :meth:`Pin.value` for more details.
      """
    def on(self) -> None:
        """
      Set pin to "1" output level.
      """
    def off(self) -> None:
        """
      Set pin to "0" output level.
      """
    def irq(
        self,
        /,
        handler: Callable[[Pin], None] | None = None,
        trigger: int = (IRQ_FALLING | IRQ_RISING),
        *,
        priority: int = 1,
        wake: int | None = None,
        hard: bool = False,
    ) -> Callable[[Pin], None] | None:
        """
      Configure an interrupt handler to be called when the trigger source of the
      pin is active.  If the pin mode is ``Pin.IN`` then the trigger source is
      the external value on the pin.  If the pin mode is ``Pin.OUT`` then the
      trigger source is the output buffer of the pin.  Otherwise, if the pin mode
      is ``Pin.OPEN_DRAIN`` then the trigger source is the output buffer for
      state '0' and the external pin value for state '1'.
      
      The arguments are:
      
        - ``handler`` is an optional function to be called when the interrupt
          triggers. The handler must take exactly one argument which is the
          ``Pin`` instance.
      
        - ``trigger`` configures the event which can generate an interrupt.
          Possible values are:
      
          - ``Pin.IRQ_FALLING`` interrupt on falling edge.
          - ``Pin.IRQ_RISING`` interrupt on rising edge.
          - ``Pin.IRQ_LOW_LEVEL`` interrupt on low level.
          - ``Pin.IRQ_HIGH_LEVEL`` interrupt on high level.
      
          These values can be OR'ed together to trigger on multiple events.
      
        - ``priority`` sets the priority level of the interrupt.  The values it
          can take are port-specific, but higher values always represent higher
          priorities.
      
        - ``wake`` selects the power mode in which this interrupt can wake up the
          system.  It can be ``machine.IDLE``, ``machine.SLEEP`` or ``machine.DEEPSLEEP``.
          These values can also be OR'ed together to make a pin generate interrupts in
          more than one power mode.
      
        - ``hard`` if true a hardware interrupt is used. This reduces the delay
          between the pin change and the handler being called. Hard interrupt
          handlers may not allocate memory; see :ref:`isr_rules`.
          Not all ports support this argument.
      
      This method returns a callback object.
      """
    def low(self) -> None:
        """
      Set pin to "0" output level.
      
      Availability: nrf, rp2, stm32 ports.
      """
    def high(self) -> None:
        """
      Set pin to "1" output level.
      
      Availability: nrf, rp2, stm32 ports.
      """
    @overload
    def mode(self) -> int:
        """
      Get or set the pin mode.
      See the constructor documentation for details of the ``mode`` argument.
      
      Availability: cc3200, stm32 ports.
      """
    @overload
    def mode(self, mode: int, /) -> None:
        """
      Get or set the pin mode.
      See the constructor documentation for details of the ``mode`` argument.
      
      Availability: cc3200, stm32 ports.
      """
    @overload
    def pull(self) -> int:
        """
      Get or set the pin pull state.
      See the constructor documentation for details of the ``pull`` argument.
      
      Availability: cc3200, stm32 ports.
      """
    @overload
    def pull(self, pull: int, /) -> None:
        """
      Get or set the pin pull state.
      See the constructor documentation for details of the ``pull`` argument.
      
      Availability: cc3200, stm32 ports.
      """
    @overload
    def dive(self) -> int:
        """
      Get or set the pin drive strength.
      See the constructor documentation for details of the ``drive`` argument.
      
      Availability: cc3200 port.
      """
    @overload
    def drive(self, drive: int, /) -> None:
        """
      Get or set the pin drive strength.
      See the constructor documentation for details of the ``drive`` argument.
      
      Availability: cc3200 port.
      """

class Signal:
    """
   The Signal class is a simple extension of the `Pin` class. Unlike Pin, which
   can be only in "absolute" 0 and 1 states, a Signal can be in "asserted"
   (on) or "deasserted" (off) states, while being inverted (active-low) or
   not. In other words, it adds logical inversion support to Pin functionality.
   While this may seem a simple addition, it is exactly what is needed to
   support wide array of simple digital devices in a way portable across
   different boards, which is one of the major MicroPython goals. Regardless
   of whether different users have an active-high or active-low LED, a normally
   open or normally closed relay - you can develop a single, nicely looking
   application which works with each of them, and capture hardware
   configuration differences in few lines in the config file of your app.
   
   Example::
   
       from machine import Pin, Signal
   
       # Suppose you have an active-high LED on pin 0
       led1_pin = Pin(0, Pin.OUT)
       # ... and active-low LED on pin 1
       led2_pin = Pin(1, Pin.OUT)
   
       # Now to light up both of them using Pin class, you'll need to set
       # them to different values
       led1_pin.value(1)
       led2_pin.value(0)
   
       # Signal class allows to abstract away active-high/active-low
       # difference
       led1 = Signal(led1_pin, invert=False)
       led2 = Signal(led2_pin, invert=True)
   
       # Now lighting up them looks the same
       led1.value(1)
       led2.value(1)
   
       # Even better:
       led1.on()
       led2.on()
   
   Following is the guide when Signal vs Pin should be used:
   
   * Use Signal: If you want to control a simple on/off (including software
     PWM!) devices like LEDs, multi-segment indicators, relays, buzzers, or
     read simple binary sensors, like normally open or normally closed buttons,
     pulled high or low, Reed switches, moisture/flame detectors, etc. etc.
     Summing up, if you have a real physical device/sensor requiring GPIO
     access, you likely should use a Signal.
   
   * Use Pin: If you implement a higher-level protocol or bus to communicate
     with more complex devices.
   
   The split between Pin and Signal come from the use cases above and the
   architecture of MicroPython: Pin offers the lowest overhead, which may
   be important when bit-banging protocols. But Signal adds additional
   flexibility on top of Pin, at the cost of minor overhead (much smaller
   than if you implemented active-high vs active-low device differences in
   Python manually!). Also, Pin is a low-level object which needs to be
   implemented for each support board, while Signal is a high-level object
   which comes for free once Pin is implemented.
   
   If in doubt, give the Signal a try! Once again, it is offered to save
   developers from the need to handle unexciting differences like active-low
   vs active-high signals, and allow other users to share and enjoy your
   application, instead of being frustrated by the fact that it doesn't
   work for them simply because their LEDs or relays are wired in a slightly
   different way.
   """

    @overload
    def __init__(self, pin_obj: Pin, invert: bool = False, /):
        """
      Create a Signal object. There're two ways to create it:
      
      * By wrapping existing Pin object - universal method which works for
        any board.
      * By passing required Pin parameters directly to Signal constructor,
        skipping the need to create intermediate Pin object. Available on
        many, but not all boards.
      
      The arguments are:
      
        - ``pin_obj`` is existing Pin object.
      
        - ``pin_arguments`` are the same arguments as can be passed to Pin constructor.
      
        - ``invert`` - if True, the signal will be inverted (active low).
      """
    @overload
    def __init__(
        self,
        id: Pin | str,
        /,
        mode: int = -1,
        pull: int = -1,
        *,
        value: Any = None,
        drive: int | None = None,
        alt: int | None = None,
        invert: bool = False,
    ):
        """
      Create a Signal object. There're two ways to create it:
      
      * By wrapping existing Pin object - universal method which works for
        any board.
      * By passing required Pin parameters directly to Signal constructor,
        skipping the need to create intermediate Pin object. Available on
        many, but not all boards.
      
      The arguments are:
      
        - ``pin_obj`` is existing Pin object.
      
        - ``pin_arguments`` are the same arguments as can be passed to Pin constructor.
      
        - ``invert`` - if True, the signal will be inverted (active low).
      """
    @overload
    def value(self) -> int:
        """
      This method allows to set and get the value of the signal, depending on whether
      the argument ``x`` is supplied or not.
      
      If the argument is omitted then this method gets the signal level, 1 meaning
      signal is asserted (active) and 0 - signal inactive.
      
      If the argument is supplied then this method sets the signal level. The
      argument ``x`` can be anything that converts to a boolean. If it converts
      to ``True``, the signal is active, otherwise it is inactive.
      
      Correspondence between signal being active and actual logic level on the
      underlying pin depends on whether signal is inverted (active-low) or not.
      For non-inverted signal, active status corresponds to logical 1, inactive -
      to logical 0. For inverted/active-low signal, active status corresponds
      to logical 0, while inactive - to logical 1.
      """
    @overload
    def value(self, x: Any, /) -> None:
        """
      This method allows to set and get the value of the signal, depending on whether
      the argument ``x`` is supplied or not.
      
      If the argument is omitted then this method gets the signal level, 1 meaning
      signal is asserted (active) and 0 - signal inactive.
      
      If the argument is supplied then this method sets the signal level. The
      argument ``x`` can be anything that converts to a boolean. If it converts
      to ``True``, the signal is active, otherwise it is inactive.
      
      Correspondence between signal being active and actual logic level on the
      underlying pin depends on whether signal is inverted (active-low) or not.
      For non-inverted signal, active status corresponds to logical 1, inactive -
      to logical 0. For inverted/active-low signal, active status corresponds
      to logical 0, while inactive - to logical 1.
      """
    def on(self) -> None:
        """
      Activate signal.
      """
    def off(self) -> None:
        """
      Deactivate signal.
      """

class ADC:
    """
   The ADC class provides an interface to analog-to-digital convertors, and
   represents a single endpoint that can sample a continuous voltage and
   convert it to a discretised value.
   
   Example usage::
   
      import machine
   
      adc = machine.ADC(pin)   # create an ADC object acting on a pin
      val = adc.read_u16()     # read a raw analog value in the range 0-65535
   """

    def __init__(self, pin: int | Pin, /):
        """
      Access the ADC associated with a source identified by *id*.  This
      *id* may be an integer (usually specifying a channel number), a
      :ref:`Pin <machine.Pin>` object, or other value supported by the
      underlying machine.
      """
    def read_u16(self) -> int:
        """
      Take an analog reading and return an integer in the range 0-65535.
      The return value represents the raw reading taken by the ADC, scaled
      such that the minimum value is 0 and the maximum value is 65535.
      """

# noinspection PyShadowingNames
class PWM:
    """
   This class provides pulse width modulation output.
   
   Example usage::
   
       from machine import PWM
   
       pwm = PWM(pin)          # create a PWM object on a pin
       pwm.duty_u16(32768)     # set duty to 50%
   
       # reinitialise with a period of 200us, duty of 5us
       pwm.init(freq=5000, duty_ns=5000)
   
       pwm.duty_ns(3000)       # set pulse width to 3us
   
       pwm.deinit()
   

   Limitations of PWM
   ------------------
   
   * Not all frequencies can be generated with absolute accuracy due to
     the discrete nature of the computing hardware.  Typically the PWM frequency
     is obtained by dividing some integer base frequency by an integer divider.
     For example, if the base frequency is 80MHz and the required PWM frequency is
     300kHz the divider must be a non-integer number 80000000 / 300000 = 266.67.
     After rounding the divider is set to 267 and the PWM frequency will be
     80000000 / 267 = 299625.5 Hz, not 300kHz.  If the divider is set to 266 then
     the PWM frequency will be 80000000 / 266 = 300751.9 Hz, but again not 300kHz.
   
   * The duty cycle has the same discrete nature and its absolute accuracy is not
     achievable.  On most hardware platforms the duty will be applied at the next
     frequency period.  Therefore, you should wait more than "1/frequency" before
     measuring the duty.
   
   * The frequency and the duty cycle resolution are usually interdependent.
     The higher the PWM frequency the lower the duty resolution which is available,
     and vice versa. For example, a 300kHz PWM frequency can have a duty cycle
     resolution of 8 bit, not 16-bit as may be expected.  In this case, the lowest
     8 bits of *duty_u16* are insignificant. So::
   
       pwm=PWM(Pin(13), freq=300_000, duty_u16=2**16//2)
   
     and::
   
       pwm=PWM(Pin(13), freq=300_000, duty_u16=2**16//2 + 255)
   
     will generate PWM with the same 50% duty cycle.
   """

    def __init__(
        self,
        dest: Pin | int,
        /,
        *,
        freq: int = ...,
        duty_u16: int = ...,
        duty_ns: int = ...,
    ):
        """
      Construct and return a new PWM object using the following parameters:
      
         - *dest* is the entity on which the PWM is output, which is usually a
           :ref:`machine.Pin <machine.Pin>` object, but a port may allow other values,
           like integers.
         - *freq* should be an integer which sets the frequency in Hz for the
           PWM cycle.
         - *duty_u16* sets the duty cycle as a ratio ``duty_u16 / 65535``.
         - *duty_ns* sets the pulse width in nanoseconds.
      
      Setting *freq* may affect other PWM objects if the objects share the same
      underlying PWM generator (this is hardware specific).
      Only one of *duty_u16* and *duty_ns* should be specified at a time.
      """
    def init(self, *, freq: int = ..., duty_u16: int = ..., duty_ns: int = ...) -> None:
        """
      Modify settings for the PWM object.  See the above constructor for details
      about the parameters.
      """
    def deinit(self) -> None:
        """
      Disable the PWM output.
      """
    @overload
    def freq(self) -> int:
        """
      Get or set the current frequency of the PWM output.
      
      With no arguments the frequency in Hz is returned.
      
      With a single *value* argument the frequency is set to that value in Hz.  The
      method may raise a ``ValueError`` if the frequency is outside the valid range.
      """
    @overload
    def freq(self, value: int, /,) -> None:
        """
      Get or set the current frequency of the PWM output.
      
      With no arguments the frequency in Hz is returned.
      
      With a single *value* argument the frequency is set to that value in Hz.  The
      method may raise a ``ValueError`` if the frequency is outside the valid range.
      """
    @overload
    def duty_u16(self) -> int:
        """
      Get or set the current duty cycle of the PWM output, as an unsigned 16-bit
      value in the range 0 to 65535 inclusive.
      
      With no arguments the duty cycle is returned.
      
      With a single *value* argument the duty cycle is set to that value, measured
      as the ratio ``value / 65535``.
      """
    @overload
    def duty_u16(self, value: int, /,) -> None:
        """
      Get or set the current duty cycle of the PWM output, as an unsigned 16-bit
      value in the range 0 to 65535 inclusive.
      
      With no arguments the duty cycle is returned.
      
      With a single *value* argument the duty cycle is set to that value, measured
      as the ratio ``value / 65535``.
      """
    @overload
    def duty_ns(self) -> int:
        """
      Get or set the current pulse width of the PWM output, as a value in nanoseconds.
      
      With no arguments the pulse width in nanoseconds is returned.
      
      With a single *value* argument the pulse width is set to that value.
      """
    @overload
    def duty_ns(self, value: int, /,) -> None:
        """
      Get or set the current pulse width of the PWM output, as a value in nanoseconds.
      
      With no arguments the pulse width in nanoseconds is returned.
      
      With a single *value* argument the pulse width is set to that value.
      """

class UART:
    """
   UART implements the standard UART/USART duplex serial communications protocol.  At
   the physical level it consists of 2 lines: RX and TX.  The unit of communication
   is a character (not to be confused with a string character) which can be 8 or 9
   bits wide.
   
   UART objects can be created and initialised using::
   
       from machine import UART
   
       uart = UART(1, 9600)                         # init with given baudrate
       uart.init(9600, bits=8, parity=None, stop=1) # init with given parameters
   
   Supported parameters differ on a board:
   
   Pyboard: Bits can be 7, 8 or 9. Stop can be 1 or 2. With *parity=None*,
   only 8 and 9 bits are supported.  With parity enabled, only 7 and 8 bits
   are supported.
   
   WiPy/CC3200: Bits can be 5, 6, 7, 8. Stop can be 1 or 2.
   
   A UART object acts like a `stream` object and reading and writing is done
   using the standard stream methods::
   
       uart.read(10)       # read 10 characters, returns a bytes object
       uart.read()         # read all available characters
       uart.readline()     # read a line
       uart.readinto(buf)  # read and store into the given buffer
       uart.write('abc')   # write the 3 characters
   """

    RX_ANY: ClassVar[int] = ...
    """
IRQ trigger sources

    Availability: WiPy.
   """
    @overload
    def __init__(
        self,
        id: int | str,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        tx: Pin | None = None,
        rx: Pin | None = None,
        txbuf: int | None = None,
        rxbuf: int | None = None,
        timeout: int | None = None,
        timeout_char: int | None = None,
        invert: int | None = None,
    ):
        """
      Construct a UART object of the given id.
      """
    @overload
    def __init__(
        self,
        id: int | str,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        pins: tuple[Pin, Pin] | None = None,
    ):
        """
      Construct a UART object of the given id.
      """
    @overload
    def __init__(
        self,
        id: int | str,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        pins: tuple[Pin, Pin, Pin, Pin] | None = None,
    ):
        """
      Construct a UART object of the given id.
      """
    @overload
    def init(
        self,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        tx: Pin | None = None,
        rx: Pin | None = None,
        txbuf: int | None = None,
        rxbuf: int | None = None,
        timeout: int | None = None,
        timeout_char: int | None = None,
        invert: int | None = None,
    ) -> None:
        """
      Initialise the UART bus with the given parameters:
      
        - *baudrate* is the clock rate.
        - *bits* is the number of bits per character, 7, 8 or 9.
        - *parity* is the parity, ``None``, 0 (even) or 1 (odd).
        - *stop* is the number of stop bits, 1 or 2.
      
      Additional keyword-only parameters that may be supported by a port are:
      
        - *tx* specifies the TX pin to use.
        - *rx* specifies the RX pin to use.
        - *rts* specifies the RTS (output) pin to use for hardware receive flow control.
        - *cts* specifies the CTS (input) pin to use for hardware transmit flow control.
        - *txbuf* specifies the length in characters of the TX buffer.
        - *rxbuf* specifies the length in characters of the RX buffer.
        - *timeout* specifies the time to wait for the first character (in ms).
        - *timeout_char* specifies the time to wait between characters (in ms).
        - *invert* specifies which lines to invert.
        - *flow* specifies which hardware flow control signals to use. The value
          is a bitmask. 
      
            - ``0`` will ignore hardware flow control signals.
            - ``UART.RTS`` will enable receive flow control by using the RTS output pin to
              signal if the receive FIFO has sufficient space to accept more data.
            - ``UART.CTS`` will enable transmit flow control by pausing transmission when the
              CTS input pin signals that the receiver is running low on buffer space.
            - ``UART.RTS | UART.CTS`` will enable both, for full hardware flow control.
      
      On the WiPy only the following keyword-only parameter is supported:
      
        - *pins* is a 4 or 2 item list indicating the TX, RX, RTS and CTS pins (in that order).
          Any of the pins can be None if one wants the UART to operate with limited functionality.
          If the RTS pin is given the the RX pin must be given as well. The same applies to CTS.
          When no pins are given, then the default set of TX and RX pins is taken, and hardware
          flow control will be disabled. If *pins* is ``None``, no pin assignment will be made.
      """
    @overload
    def init(
        self,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        pins: tuple[Pin, Pin] | None = None,
    ) -> None:
        """
      Initialise the UART bus with the given parameters:
      
        - *baudrate* is the clock rate.
        - *bits* is the number of bits per character, 7, 8 or 9.
        - *parity* is the parity, ``None``, 0 (even) or 1 (odd).
        - *stop* is the number of stop bits, 1 or 2.
      
      Additional keyword-only parameters that may be supported by a port are:
      
        - *tx* specifies the TX pin to use.
        - *rx* specifies the RX pin to use.
        - *rts* specifies the RTS (output) pin to use for hardware receive flow control.
        - *cts* specifies the CTS (input) pin to use for hardware transmit flow control.
        - *txbuf* specifies the length in characters of the TX buffer.
        - *rxbuf* specifies the length in characters of the RX buffer.
        - *timeout* specifies the time to wait for the first character (in ms).
        - *timeout_char* specifies the time to wait between characters (in ms).
        - *invert* specifies which lines to invert.
        - *flow* specifies which hardware flow control signals to use. The value
          is a bitmask. 
      
            - ``0`` will ignore hardware flow control signals.
            - ``UART.RTS`` will enable receive flow control by using the RTS output pin to
              signal if the receive FIFO has sufficient space to accept more data.
            - ``UART.CTS`` will enable transmit flow control by pausing transmission when the
              CTS input pin signals that the receiver is running low on buffer space.
            - ``UART.RTS | UART.CTS`` will enable both, for full hardware flow control.
      
      On the WiPy only the following keyword-only parameter is supported:
      
        - *pins* is a 4 or 2 item list indicating the TX, RX, RTS and CTS pins (in that order).
          Any of the pins can be None if one wants the UART to operate with limited functionality.
          If the RTS pin is given the the RX pin must be given as well. The same applies to CTS.
          When no pins are given, then the default set of TX and RX pins is taken, and hardware
          flow control will be disabled. If *pins* is ``None``, no pin assignment will be made.
      """
    @overload
    def init(
        self,
        baudrate: int = 9600,
        bits: int = 8,
        parity: int | None = None,
        stop: int = 1,
        /,
        *,
        pins: tuple[Pin, Pin, Pin, Pin] | None = None,
    ) -> None:
        """
      Initialise the UART bus with the given parameters:
      
        - *baudrate* is the clock rate.
        - *bits* is the number of bits per character, 7, 8 or 9.
        - *parity* is the parity, ``None``, 0 (even) or 1 (odd).
        - *stop* is the number of stop bits, 1 or 2.
      
      Additional keyword-only parameters that may be supported by a port are:
      
        - *tx* specifies the TX pin to use.
        - *rx* specifies the RX pin to use.
        - *rts* specifies the RTS (output) pin to use for hardware receive flow control.
        - *cts* specifies the CTS (input) pin to use for hardware transmit flow control.
        - *txbuf* specifies the length in characters of the TX buffer.
        - *rxbuf* specifies the length in characters of the RX buffer.
        - *timeout* specifies the time to wait for the first character (in ms).
        - *timeout_char* specifies the time to wait between characters (in ms).
        - *invert* specifies which lines to invert.
        - *flow* specifies which hardware flow control signals to use. The value
          is a bitmask. 
      
            - ``0`` will ignore hardware flow control signals.
            - ``UART.RTS`` will enable receive flow control by using the RTS output pin to
              signal if the receive FIFO has sufficient space to accept more data.
            - ``UART.CTS`` will enable transmit flow control by pausing transmission when the
              CTS input pin signals that the receiver is running low on buffer space.
            - ``UART.RTS | UART.CTS`` will enable both, for full hardware flow control.
      
      On the WiPy only the following keyword-only parameter is supported:
      
        - *pins* is a 4 or 2 item list indicating the TX, RX, RTS and CTS pins (in that order).
          Any of the pins can be None if one wants the UART to operate with limited functionality.
          If the RTS pin is given the the RX pin must be given as well. The same applies to CTS.
          When no pins are given, then the default set of TX and RX pins is taken, and hardware
          flow control will be disabled. If *pins* is ``None``, no pin assignment will be made.
      """
    def deinit(self) -> None:
        """
      Turn off the UART bus.
      """
    def any(self) -> int:
        """
      Returns an integer counting the number of characters that can be read without
      blocking.  It will return 0 if there are no characters available and a positive
      number if there are characters.  The method may return 1 even if there is more
      than one character available for reading.
      
      For more sophisticated querying of available characters use select.poll::
      
       poll = select.poll()
       poll.register(uart, select.POLLIN)
       poll.poll(timeout)
      """
    @overload
    def read(self) -> bytes | None:
        """
      Read characters.  If ``nbytes`` is specified then read at most that many bytes,
      otherwise read as much data as possible. It may return sooner if a timeout
      is reached. The timeout is configurable in the constructor.
      
      Return value: a bytes object containing the bytes read in.  Returns ``None``
      on timeout.
      """
    @overload
    def read(self, nbytes: int, /) -> bytes | None:
        """
      Read characters.  If ``nbytes`` is specified then read at most that many bytes,
      otherwise read as much data as possible. It may return sooner if a timeout
      is reached. The timeout is configurable in the constructor.
      
      Return value: a bytes object containing the bytes read in.  Returns ``None``
      on timeout.
      """
    @overload
    def readinto(self, buf: AnyWritableBuf, /) -> int | None:
        """
      Read bytes into the ``buf``.  If ``nbytes`` is specified then read at most
      that many bytes.  Otherwise, read at most ``len(buf)`` bytes. It may return sooner if a timeout
      is reached. The timeout is configurable in the constructor.
      
      Return value: number of bytes read and stored into ``buf`` or ``None`` on
      timeout.
      """
    @overload
    def readinto(self, buf: AnyWritableBuf, nbytes: int, /) -> int | None:
        """
      Read bytes into the ``buf``.  If ``nbytes`` is specified then read at most
      that many bytes.  Otherwise, read at most ``len(buf)`` bytes. It may return sooner if a timeout
      is reached. The timeout is configurable in the constructor.
      
      Return value: number of bytes read and stored into ``buf`` or ``None`` on
      timeout.
      """
    def readline(self) -> bytes | None:
        """
      Read a line, ending in a newline character. It may return sooner if a timeout
      is reached. The timeout is configurable in the constructor.
      
      Return value: the line read or ``None`` on timeout.
      """
    def write(self, buf: AnyReadableBuf, /) -> int | None:
        """
      Write the buffer of bytes to the bus.
      
      Return value: number of bytes written or ``None`` on timeout.
      """
    def sendbreak(self) -> None:
        """
      Send a break condition on the bus. This drives the bus low for a duration
      longer than required for a normal transmission of a character.
      """
    def irq(
        self,
        trigger: int,
        priority: int = 1,
        handler: Callable[[UART], None] | None = None,
        wake: int = IDLE,
        /,
    ) -> Any:
        """
      Create a callback to be triggered when data is received on the UART.
      
          - *trigger* can only be ``UART.RX_ANY``
          - *priority* level of the interrupt. Can take values in the range 1-7.
            Higher values represent higher priorities.
          - *handler* an optional function to be called when new characters arrive.
          - *wake* can only be ``machine.IDLE``.
      
      .. note::
      
         The handler will be called whenever any of the following two conditions are met:
      
             - 8 new characters have been received.
             - At least 1 new character is waiting in the Rx buffer and the Rx line has been
               silent for the duration of 1 complete frame.
      
         This means that when the handler function is called there will be between 1 to 8
         characters waiting.
      
      Returns an irq object.
      
      Availability: WiPy.
      """

class SPI:
    """
   SPI is a synchronous serial protocol that is driven by a controller. At the
   physical level, a bus consists of 3 lines: SCK, MOSI, MISO. Multiple devices
   can share the same bus. Each device should have a separate, 4th signal,
   CS (Chip Select), to select a particular device on a bus with which
   communication takes place. Management of a CS signal should happen in
   user code (via machine.Pin class).
   
   Both hardware and software SPI implementations exist via the
   :ref:`machine.SPI <machine.SPI>` and `machine.SoftSPI` classes.  Hardware SPI uses underlying
   hardware support of the system to perform the reads/writes and is usually
   efficient and fast but may have restrictions on which pins can be used.
   Software SPI is implemented by bit-banging and can be used on any pin but
   is not as efficient.  These classes have the same methods available and
   differ primarily in the way they are constructed.
   
   Example usage::
   
       from machine import SPI, Pin
   
       spi = SPI(0, baudrate=400000)           # Create SPI peripheral 0 at frequency of 400kHz.
                                               # Depending on the use case, extra parameters may be required
                                               # to select the bus characteristics and/or pins to use.
       cs = Pin(4, mode=Pin.OUT, value=1)      # Create chip-select on pin 4.
   
       try:
           cs(0)                               # Select peripheral.
           spi.write(b"12345678")              # Write 8 bytes, and don't care about received data.
       finally:
           cs(1)                               # Deselect peripheral.
   
       try:
           cs(0)                               # Select peripheral.
           rxdata = spi.read(8, 0x42)          # Read 8 bytes while writing 0x42 for each byte.
       finally:
           cs(1)                               # Deselect peripheral.
   
       rxdata = bytearray(8)
       try:
           cs(0)                               # Select peripheral.
           spi.readinto(rxdata, 0x42)          # Read 8 bytes inplace while writing 0x42 for each byte.
       finally:
           cs(1)                               # Deselect peripheral.
   
       txdata = b"12345678"
       rxdata = bytearray(len(txdata))
       try:
           cs(0)                               # Select peripheral.
           spi.write_readinto(txdata, rxdata)  # Simultaneously write and read bytes.
       finally:
           cs(1)                               # Deselect peripheral.
   """

    CONTROLLER: ClassVar[int] = ...
    """
for initialising the SPI bus to controller; this is only used for the WiPy
   """

    MSB: ClassVar[int] = ...
    """
set the first bit to be the most significant bit
   """

    LSB: ClassVar[int] = ...
    """
set the first bit to be the least significant bit
   """
    @overload
    def __init__(self, id: int, /):
        """
      Construct an SPI object on the given bus, *id*. Values of *id* depend
      on a particular port and its hardware. Values 0, 1, etc. are commonly used
      to select hardware SPI block #0, #1, etc.
      
      With no additional parameters, the SPI object is created but not
      initialised (it has the settings from the last initialisation of
      the bus, if any).  If extra arguments are given, the bus is initialised.
      See ``init`` for parameters of initialisation.
      """
    @overload
    def __init__(
        self,
        id: int,
        /,
        baudrate: int = 1_000_000,
        *,
        polarity: int = 0,
        phase: int = 0,
        bits: int = 8,
        firstbit: int = MSB,
        sck: Pin | None = None,
        mosi: Pin | None = None,
        miso: Pin | None = None,
    ):
        """
      Construct an SPI object on the given bus, *id*. Values of *id* depend
      on a particular port and its hardware. Values 0, 1, etc. are commonly used
      to select hardware SPI block #0, #1, etc.
      
      With no additional parameters, the SPI object is created but not
      initialised (it has the settings from the last initialisation of
      the bus, if any).  If extra arguments are given, the bus is initialised.
      See ``init`` for parameters of initialisation.
      """
    @overload
    def __init__(
        self,
        id: int,
        /,
        baudrate: int = 1_000_000,
        *,
        polarity: int = 0,
        phase: int = 0,
        bits: int = 8,
        firstbit: int = MSB,
        pins: tuple[Pin, Pin, Pin] | None = None,
    ):
        """
      Construct an SPI object on the given bus, *id*. Values of *id* depend
      on a particular port and its hardware. Values 0, 1, etc. are commonly used
      to select hardware SPI block #0, #1, etc.
      
      With no additional parameters, the SPI object is created but not
      initialised (it has the settings from the last initialisation of
      the bus, if any).  If extra arguments are given, the bus is initialised.
      See ``init`` for parameters of initialisation.
      """
    @overload
    def init(
        self,
        baudrate: int = 1_000_000,
        *,
        polarity: int = 0,
        phase: int = 0,
        bits: int = 8,
        firstbit: int = MSB,
        sck: Pin | None = None,
        mosi: Pin | None = None,
        miso: Pin | None = None,
    ) -> None:
        """
      Initialise the SPI bus with the given parameters:
      
        - ``baudrate`` is the SCK clock rate.
        - ``polarity`` can be 0 or 1, and is the level the idle clock line sits at.
        - ``phase`` can be 0 or 1 to sample data on the first or second clock edge
          respectively.
        - ``bits`` is the width in bits of each transfer. Only 8 is guaranteed to be supported by all hardware.
        - ``firstbit`` can be ``SPI.MSB`` or ``SPI.LSB``.
        - ``sck``, ``mosi``, ``miso`` are pins (machine.Pin) objects to use for bus signals. For most
          hardware SPI blocks (as selected by ``id`` parameter to the constructor), pins are fixed
          and cannot be changed. In some cases, hardware blocks allow 2-3 alternative pin sets for
          a hardware SPI block. Arbitrary pin assignments are possible only for a bitbanging SPI driver
          (``id`` = -1).
        - ``pins`` - WiPy port doesn't ``sck``, ``mosi``, ``miso`` arguments, and instead allows to
          specify them as a tuple of ``pins`` parameter.
      
      In the case of hardware SPI the actual clock frequency may be lower than the
      requested baudrate. This is dependant on the platform hardware. The actual
      rate may be determined by printing the SPI object.
      """
    @overload
    def init(
        self,
        baudrate: int = 1_000_000,
        *,
        polarity: int = 0,
        phase: int = 0,
        bits: int = 8,
        firstbit: int = MSB,
        pins: tuple[Pin, Pin, Pin] | None = None,
    ) -> None:
        """
      Initialise the SPI bus with the given parameters:
      
        - ``baudrate`` is the SCK clock rate.
        - ``polarity`` can be 0 or 1, and is the level the idle clock line sits at.
        - ``phase`` can be 0 or 1 to sample data on the first or second clock edge
          respectively.
        - ``bits`` is the width in bits of each transfer. Only 8 is guaranteed to be supported by all hardware.
        - ``firstbit`` can be ``SPI.MSB`` or ``SPI.LSB``.
        - ``sck``, ``mosi``, ``miso`` are pins (machine.Pin) objects to use for bus signals. For most
          hardware SPI blocks (as selected by ``id`` parameter to the constructor), pins are fixed
          and cannot be changed. In some cases, hardware blocks allow 2-3 alternative pin sets for
          a hardware SPI block. Arbitrary pin assignments are possible only for a bitbanging SPI driver
          (``id`` = -1).
        - ``pins`` - WiPy port doesn't ``sck``, ``mosi``, ``miso`` arguments, and instead allows to
          specify them as a tuple of ``pins`` parameter.
      
      In the case of hardware SPI the actual clock frequency may be lower than the
      requested baudrate. This is dependant on the platform hardware. The actual
      rate may be determined by printing the SPI object.
      """
    def deinit(self) -> None:
        """
      Turn off the SPI bus.
      """
    def read(self, nbytes: int, write: int = 0x00, /) -> bytes:
        """
       Read a number of bytes specified by ``nbytes`` while continuously writing
       the single byte given by ``write``.
       Returns a ``bytes`` object with the data that was read.
      """
    def readinto(self, buf: AnyWritableBuf, write: int = 0x00, /) -> int | None:
        """
       Read into the buffer specified by ``buf`` while continuously writing the
       single byte given by ``write``.
       Returns ``None``.
       
       Note: on WiPy this function returns the number of bytes read.
      """
    def write(self, buf: AnyReadableBuf, /) -> int | None:
        """
       Write the bytes contained in ``buf``.
       Returns ``None``.
       
       Note: on WiPy this function returns the number of bytes written.
      """
    def write_readinto(
        self, write_buf: AnyReadableBuf, read_buf: AnyWritableBuf, /
    ) -> int | None:
        """
       Write the bytes from ``write_buf`` while reading into ``read_buf``.  The
       buffers can be the same or different, but both buffers must have the
       same length.
       Returns ``None``.
       
       Note: on WiPy this function returns the number of bytes written.
      """

# noinspection PyShadowingNames
class I2C:
    """
   I2C is a two-wire protocol for communicating between devices.  At the physical
   level it consists of 2 wires: SCL and SDA, the clock and data lines respectively.
   
   I2C objects are created attached to a specific bus.  They can be initialised
   when created, or initialised later on.
   
   Printing the I2C object gives you information about its configuration.
   
   Both hardware and software I2C implementations exist via the
   :ref:`machine.I2C <machine.I2C>` and `machine.SoftI2C` classes.  Hardware I2C uses
   underlying hardware support of the system to perform the reads/writes and is
   usually efficient and fast but may have restrictions on which pins can be used.
   Software I2C is implemented by bit-banging and can be used on any pin but is not
   as efficient.  These classes have the same methods available and differ primarily
   in the way they are constructed.
   
   Example usage::
   
       from machine import I2C
   
       i2c = I2C(freq=400000)          # create I2C peripheral at frequency of 400kHz
                                       # depending on the port, extra parameters may be required
                                       # to select the peripheral and/or pins to use
   
       i2c.scan()                      # scan for peripherals, returning a list of 7-bit addresses
   
       i2c.writeto(42, b'123')         # write 3 bytes to peripheral with 7-bit address 42
       i2c.readfrom(42, 4)             # read 4 bytes from peripheral with 7-bit address 42
   
       i2c.readfrom_mem(42, 8, 3)      # read 3 bytes from memory of peripheral 42,
                                       #   starting at memory-address 8 in the peripheral
       i2c.writeto_mem(42, 2, b'\x10') # write 1 byte to memory of peripheral 42
                                       #   starting at address 2 in the peripheral
   """

    @overload
    def __init__(self, id: int, /, *, freq: int = 400_000):
        """
      Construct and return a new I2C object using the following parameters:
      
         - *id* identifies a particular I2C peripheral.  Allowed values for
           depend on the particular port/board
         - *scl* should be a pin object specifying the pin to use for SCL.
         - *sda* should be a pin object specifying the pin to use for SDA.
         - *freq* should be an integer which sets the maximum frequency
           for SCL.
      
      Note that some ports/boards will have default values of *scl* and *sda*
      that can be changed in this constructor.  Others will have fixed values
      of *scl* and *sda* that cannot be changed.
      """
    @overload
    def __init__(self, id: int, /, *, scl: Pin, sda: Pin, freq: int = 400_000):
        """
      Construct and return a new I2C object using the following parameters:
      
         - *id* identifies a particular I2C peripheral.  Allowed values for
           depend on the particular port/board
         - *scl* should be a pin object specifying the pin to use for SCL.
         - *sda* should be a pin object specifying the pin to use for SDA.
         - *freq* should be an integer which sets the maximum frequency
           for SCL.
      
      Note that some ports/boards will have default values of *scl* and *sda*
      that can be changed in this constructor.  Others will have fixed values
      of *scl* and *sda* that cannot be changed.
      """
    @overload
    def init(self, *, freq: int = 400_000) -> None:
        """
     Initialise the I2C bus with the given arguments:
     
        - *scl* is a pin object for the SCL line
        - *sda* is a pin object for the SDA line
        - *freq* is the SCL clock rate
      """
    @overload
    def init(self, *, scl: Pin, sda: Pin, freq: int = 400_000) -> None:
        """
     Initialise the I2C bus with the given arguments:
     
        - *scl* is a pin object for the SCL line
        - *sda* is a pin object for the SDA line
        - *freq* is the SCL clock rate
      """
    def deinit(self) -> None:
        """
      Turn off the I2C bus.
      
      Availability: WiPy.
      """
    def scan(self) -> list[int]:
        """
      Scan all I2C addresses between 0x08 and 0x77 inclusive and return a list of
      those that respond.  A device responds if it pulls the SDA line low after
      its address (including a write bit) is sent on the bus.
      """
    def start(self) -> None:
        """
      Generate a START condition on the bus (SDA transitions to low while SCL is high).
      
      
      Primitive I2C operations
      ------------------------
      
      The following methods implement the primitive I2C controller bus operations and can
      be combined to make any I2C transaction.  They are provided if you need more
      control over the bus, otherwise the standard methods (see below) can be used.
      
      These methods are only available on the `machine.SoftI2C` class.
      """
    def stop(self) -> None:
        """
      Generate a STOP condition on the bus (SDA transitions to high while SCL is high).
      
      
      Primitive I2C operations
      ------------------------
      
      The following methods implement the primitive I2C controller bus operations and can
      be combined to make any I2C transaction.  They are provided if you need more
      control over the bus, otherwise the standard methods (see below) can be used.
      
      These methods are only available on the `machine.SoftI2C` class.
      """
    def readinto(self, buf: AnyWritableBuf, nack: bool = True, /) -> None:
        """
      Reads bytes from the bus and stores them into *buf*.  The number of bytes
      read is the length of *buf*.  An ACK will be sent on the bus after
      receiving all but the last byte.  After the last byte is received, if *nack*
      is true then a NACK will be sent, otherwise an ACK will be sent (and in this
      case the peripheral assumes more bytes are going to be read in a later call).
      
      
      Primitive I2C operations
      ------------------------
      
      The following methods implement the primitive I2C controller bus operations and can
      be combined to make any I2C transaction.  They are provided if you need more
      control over the bus, otherwise the standard methods (see below) can be used.
      
      These methods are only available on the `machine.SoftI2C` class.
      """
    def write(self, buf: AnyReadableBuf, /) -> int:
        """
      Write the bytes from *buf* to the bus.  Checks that an ACK is received
      after each byte and stops transmitting the remaining bytes if a NACK is
      received.  The function returns the number of ACKs that were received.
      
      
      Primitive I2C operations
      ------------------------
      
      The following methods implement the primitive I2C controller bus operations and can
      be combined to make any I2C transaction.  They are provided if you need more
      control over the bus, otherwise the standard methods (see below) can be used.
      
      These methods are only available on the `machine.SoftI2C` class.
      """
    def readfrom(self, addr: int, nbytes: int, stop: bool = True, /) -> bytes:
        """
      Read *nbytes* from the peripheral specified by *addr*.
      If *stop* is true then a STOP condition is generated at the end of the transfer.
      Returns a `bytes` object with the data read.
      
      
      Standard bus operations
      -----------------------
      
      The following methods implement the standard I2C controller read and write
      operations that target a given peripheral device.
      """
    def readfrom_into(
        self, addr: int, buf: AnyWritableBuf, stop: bool = True, /
    ) -> None:
        """
      Read into *buf* from the peripheral specified by *addr*.
      The number of bytes read will be the length of *buf*.
      If *stop* is true then a STOP condition is generated at the end of the transfer.
      
      The method returns ``None``.
      
      
      Standard bus operations
      -----------------------
      
      The following methods implement the standard I2C controller read and write
      operations that target a given peripheral device.
      """
    def writeto(self, addr: int, buf: AnyReadableBuf, stop: bool = True, /) -> int:
        """
      Write the bytes from *buf* to the peripheral specified by *addr*.  If a
      NACK is received following the write of a byte from *buf* then the
      remaining bytes are not sent.  If *stop* is true then a STOP condition is
      generated at the end of the transfer, even if a NACK is received.
      The function returns the number of ACKs that were received.
      
      
      Standard bus operations
      -----------------------
      
      The following methods implement the standard I2C controller read and write
      operations that target a given peripheral device.
      """
    def writevto(
        self, addr: int, vector: Sequence[AnyReadableBuf], stop: bool = True, /
    ) -> int:
        """
      Write the bytes contained in *vector* to the peripheral specified by *addr*.
      *vector* should be a tuple or list of objects with the buffer protocol.
      The *addr* is sent once and then the bytes from each object in *vector*
      are written out sequentially.  The objects in *vector* may be zero bytes
      in length in which case they don't contribute to the output.
      
      If a NACK is received following the write of a byte from one of the
      objects in *vector* then the remaining bytes, and any remaining objects,
      are not sent.  If *stop* is true then a STOP condition is generated at
      the end of the transfer, even if a NACK is received.  The function
      returns the number of ACKs that were received.
      
      
      Standard bus operations
      -----------------------
      
      The following methods implement the standard I2C controller read and write
      operations that target a given peripheral device.
      """
    def readfrom_mem(
        self, addr: int, memaddr: int, nbytes: int, /, *, addrsize: int = 8
    ) -> bytes:
        """
      Read *nbytes* from the peripheral specified by *addr* starting from the memory
      address specified by *memaddr*.
      The argument *addrsize* specifies the address size in bits.
      Returns a `bytes` object with the data read.
      
      
      Memory operations
      -----------------
      
      Some I2C devices act as a memory device (or set of registers) that can be read
      from and written to.  In this case there are two addresses associated with an
      I2C transaction: the peripheral address and the memory address.  The following
      methods are convenience functions to communicate with such devices.
      """
    def readfrom_mem_into(
        self, addr: int, memaddr: int, buf: AnyWritableBuf, /, *, addrsize: int = 8
    ) -> None:
        """
      Read into *buf* from the peripheral specified by *addr* starting from the
      memory address specified by *memaddr*.  The number of bytes read is the
      length of *buf*.
      The argument *addrsize* specifies the address size in bits (on ESP8266
      this argument is not recognised and the address size is always 8 bits).
      
      The method returns ``None``.
      
      
      Memory operations
      -----------------
      
      Some I2C devices act as a memory device (or set of registers) that can be read
      from and written to.  In this case there are two addresses associated with an
      I2C transaction: the peripheral address and the memory address.  The following
      methods are convenience functions to communicate with such devices.
      """
    def writeto_mem(
        self, addr: int, memaddr: int, buf: AnyReadableBuf, /, *, addrsize: int = 8
    ) -> None:
        """
      Write *buf* to the peripheral specified by *addr* starting from the
      memory address specified by *memaddr*.
      The argument *addrsize* specifies the address size in bits (on ESP8266
      this argument is not recognised and the address size is always 8 bits).
      
      The method returns ``None``.
      
      Memory operations
      -----------------
      
      Some I2C devices act as a memory device (or set of registers) that can be read
      from and written to.  In this case there are two addresses associated with an
      I2C transaction: the peripheral address and the memory address.  The following
      methods are convenience functions to communicate with such devices.
      """

class I2S:
    """
   I2S is a synchronous serial protocol used to connect digital audio devices. 
   At the physical level, a bus consists of 3 lines: SCK, WS, SD.
   The I2S class supports controller operation.  Peripheral operation is not supported.
   
   The I2S class is currently available as a Technical Preview.  During the preview period, feedback from 
   users is encouraged.  Based on this feedback, the I2S class API and implementation may be changed.
   
   I2S objects can be created and initialized using::
   
       from machine import I2S
       from machine import Pin
       
       # ESP32
       sck_pin = Pin(14)   # Serial clock output
       ws_pin = Pin(13)    # Word clock output
       sd_pin = Pin(12)    # Serial data output
       
       or
       
       # PyBoards
       sck_pin = Pin("Y6")   # Serial clock output
       ws_pin = Pin("Y5")    # Word clock output
       sd_pin = Pin("Y8")    # Serial data output
       
       audio_out = I2S(2, 
                       sck=sck_pin, ws=ws_pin, sd=sd_pin,
                       mode=I2S.TX, 
                       bits=16,                       
                       format=I2S.MONO,
                       rate=44100, 
                       ibuf=20000)
                      
       audio_in = I2S(2, 
                      sck=sck_pin, ws=ws_pin, sd=sd_pin,
                      mode=I2S.RX, 
                      bits=32,                       
                      format=I2S.STEREO,
                      rate=22050, 
                      ibuf=20000)
                       
   3 modes of operation are supported:
    - blocking 
    - non-blocking 
    - uasyncio
     
   blocking::
    
      num_written = audio_out.write(buf) # blocks until buf emptied
   
      num_read = audio_in.readinto(buf) # blocks until buf filled
      
   non-blocking::
    
      audio_out.irq(i2s_callback)         # i2s_callback is called when buf is emptied
      num_written = audio_out.write(buf)  # returns immediately
           
      audio_in.irq(i2s_callback)          # i2s_callback is called when buf is filled
      num_read = audio_in.readinto(buf)   # returns immediately    
    
   uasyncio::
    
      swriter = uasyncio.StreamWriter(audio_out)
      swriter.write(buf)
      await swriter.drain()
      
      sreader = uasyncio.StreamReader(audio_in)
      num_read = await sreader.readinto(buf)
   """

    RX: ClassVar[int] = ...
    """
for initialising the I2S bus ``mode`` to receive
   """

    TX: ClassVar[int] = ...
    """
for initialising the I2S bus ``mode`` to transmit
   """

    STEREO: ClassVar[int] = ...
    """
for initialising the I2S bus ``format`` to stereo
   """

    MONO: ClassVar[int] = ...
    """
for initialising the I2S bus ``format`` to mono
   """
    def __init__(
        self,
        id: int,
        /,
        *,
        sck: Pin,
        ws: Pin,
        sd: Pin,
        mode: int,
        bits: int,
        format: int,
        rate: int,
        ibuf: int,
    ):
        """
      Construct an I2S object of the given id:
      
      - ``id`` identifies a particular I2S bus.  
      
      ``id`` is board and port specific:
      
        - PYBv1.0/v1.1: has one I2S bus with id=2.
        - PYBD-SFxW: has two I2S buses with id=1 and id=2. 
        - ESP32: has two I2S buses with id=0 and id=1. 
      
      Keyword-only parameters that are supported on all ports:
       
        - ``sck`` is a pin object for the serial clock line
        - ``ws`` is a pin object for the word select line
        - ``sd`` is a pin object for the serial data line
        - ``mode`` specifies receive or transmit
        - ``bits`` specifies sample size (bits), 16 or 32
        - ``format`` specifies channel format, STEREO or MONO
        - ``rate`` specifies audio sampling rate (samples/s)
        - ``ibuf`` specifies internal buffer length (bytes)
        
      For all ports, DMA runs continuously in the background and allows user applications to perform other operations while 
      sample data is transfered between the internal buffer and the I2S peripheral unit. 
      Increasing the size of the internal buffer has the potential to increase the time that user applications can perform non-I2S operations
      before underflow (e.g. ``write`` method) or overflow (e.g. ``readinto`` method).
      """
    def init(
        self,
        *,
        sck: Pin,
        ws: Pin,
        sd: Pin,
        mode: int,
        bits: int,
        format: int,
        rate: int,
        ibuf: int,
    ) -> None:
        """
     see Constructor for argument descriptions
      """
    def deinit(self) -> None:
        """
     Deinitialize the I2S bus
      """
    def readinto(self, buf: AnyWritableBuf, /,) -> int:
        """
     Read audio samples into the buffer specified by ``buf``.  ``buf`` must support the buffer protocol, such as bytearray or array. 
     "buf" byte ordering is little-endian.  For Stereo format, left channel sample precedes right channel sample. For Mono format, 
     the left channel sample data is used.
     Returns number of bytes read
      """
    def write(self, buf: AnyReadableBuf, /,) -> int:
        """
     Write audio samples contained in ``buf``. ``buf`` must support the buffer protocol, such as bytearray or array.
     "buf" byte ordering is little-endian.  For Stereo format, left channel sample precedes right channel sample. For Mono format, 
     the sample data is written to both the right and left channels.
     Returns number of bytes written
      """
    def irq(self, handler: Callable[[], None], /,) -> None:
        """
     Set a callback. ``handler`` is called when ``buf`` is emptied (``write`` method) or becomes full (``readinto`` method).  
     Setting a callback changes the ``write`` and ``readinto`` methods to non-blocking operation.
     ``handler`` is called in the context of the MicroPython scheduler.
      """
    @staticmethod
    def shift(buf: AnyWritableBuf, bits: int, shift: int, /,) -> None:
        """
     bitwise shift of all samples contained in ``buf``. ``bits`` specifies sample size in bits. ``shift`` specifies the number of bits to shift each sample. 
     Positive for left shift, negative for right shift. 
     Typically used for volume control.  Each bit shift changes sample volume by 6dB.
      """

class RTC:
    """
   The RTC is an independent clock that keeps track of the date
   and time.
   
   Example usage::
   
       rtc = machine.RTC()
       rtc.datetime((2020, 1, 21, 2, 10, 32, 36, 0))
       print(rtc.datetime())
   
   

   The documentation for RTC is in a poor state; better to experiment and use `dir`!
   """

    ALARM0: ClassVar[int] = ...
    """
irq trigger source
   The documentation for RTC is in a poor state; better to experiment and use `dir`!
   """
    @overload
    def __init__(self, id: int = 0, /, *, datetime: tuple[int, int, int]):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def __init__(self, id: int = 0, /, *, datetime: tuple[int, int, int, int]):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def __init__(self, id: int = 0, /, *, datetime: tuple[int, int, int, int, int]):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def __init__(
        self, id: int = 0, /, *, datetime: tuple[int, int, int, int, int, int]
    ):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def __init__(
        self, id: int = 0, /, *, datetime: tuple[int, int, int, int, int, int, int]
    ):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def __init__(
        self, id: int = 0, /, *, datetime: tuple[int, int, int, int, int, int, int, int]
    ):
        """
      Create an RTC object. See init for parameters of initialization.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int, int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int, int, int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def init(self, datetime: tuple[int, int, int, int, int, int, int, int], /) -> None:
        """
      Initialise the RTC. Datetime is a tuple of the form:
      
         ``(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])``
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    def now(self) -> tuple[int, int, int, int, int, int, int, int]:
        """
      Get get the current datetime tuple.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    def deinit(self) -> None:
        """
      Resets the RTC to the time of January 1, 2015 and starts running it again.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: int, /, *, repeat: bool = False) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: tuple[int, int, int], /) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: tuple[int, int, int, int], /) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: tuple[int, int, int, int, int], /) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: tuple[int, int, int, int, int, int], /) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(self, id: int, time: tuple[int, int, int, int, int, int, int], /) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    @overload
    def alarm(
        self, id: int, time: tuple[int, int, int, int, int, int, int, int], /
    ) -> None:
        """
      Set the RTC alarm. Time might be either a millisecond value to program the alarm to
      current time + time_in_ms in the future, or a datetimetuple. If the time passed is in
      milliseconds, repeat can be set to ``True`` to make the alarm periodic.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    def alarm_left(self, alarm_id: int = 0, /) -> int:
        """
      Get the number of milliseconds left before the alarm expires.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    def cancel(self, alarm_id: int = 0, /) -> None:
        """
      Cancel a running alarm.
      
      The documentation for RTC is in a poor state; better to experiment and use `dir`!
      """
    def irq(
        self,
        /,
        *,
        trigger: int,
        handler: Callable[[RTC], None] | None = None,
        wake: int = IDLE,
    ) -> None:
        """
      Create an irq object triggered by a real time clock alarm.
      
         - ``trigger`` must be ``RTC.ALARM0``
         - ``handler`` is the function to be called when the callback is triggered.
         - ``wake`` specifies the sleep mode from where this interrupt can wake
           up the system.
      """

class Timer:
    """
   Hardware timers deal with timing of periods and events. Timers are perhaps
   the most flexible and heterogeneous kind of hardware in MCUs and SoCs,
   differently greatly from a model to a model. MicroPython's Timer class
   defines a baseline operation of executing a callback with a given period
   (or once after some delay), and allow specific boards to define more
   non-standard behaviour (which thus won't be portable to other boards).
   
   See discussion of :ref:`important constraints <machine_callbacks>` on
   Timer callbacks.
   
   .. note::
   
       Memory can't be allocated inside irq handlers (an interrupt) and so
       exceptions raised within a handler don't give much information.  See
       :func:`micropython.alloc_emergency_exception_buf` for how to get around this
       limitation.
   
   If you are using a WiPy board please refer to :ref:`machine.TimerWiPy <machine.TimerWiPy>`
   instead of this class.
   """

    ONE_SHOT: ClassVar[int] = ...
    """
Timer operating mode.
   """

    PERIODIC: ClassVar[int] = ...
    """
Timer operating mode.
   """
    @overload
    def __init__(self, id: int, /):
        """
      Construct a new timer object of the given ``id``. ``id`` of -1 constructs a
      virtual timer (if supported by a board).
      ``id`` shall not be passed as a keyword argument.
      
      See ``init`` for parameters of initialisation.
      """
    @overload
    def __init__(
        self,
        id: int,
        /,
        *,
        mode: int = PERIODIC,
        period: int = -1,
        callback: Callable[[Timer], None] | None = None,
    ):
        """
      Construct a new timer object of the given ``id``. ``id`` of -1 constructs a
      virtual timer (if supported by a board).
      ``id`` shall not be passed as a keyword argument.
      
      See ``init`` for parameters of initialisation.
      """
    def init(
        self,
        *,
        mode: int = PERIODIC,
        period: int = -1,
        callback: Callable[[Timer], None] | None = None,
    ) -> None:
        """
      Initialise the timer. Example::
      
          def mycallback(t):
              pass
      
          # periodic with 100ms period
          tim.init(period=100, callback=mycallback)
      
          # one shot firing after 1000ms
          tim.init(mode=Timer.ONE_SHOT, period=1000, callback=mycallback)
      
      Keyword arguments:
      
        - ``mode`` can be one of:
      
          - ``Timer.ONE_SHOT`` - The timer runs once until the configured
            period of the channel expires.
          - ``Timer.PERIODIC`` - The timer runs periodically at the configured
            frequency of the channel.
      
        - ``period`` - The timer period, in milliseconds.
      
        - ``callback`` - The callable to call upon expiration of the timer period.
          The callback must take one argument, which is passed the Timer object.
          The ``callback`` argument shall be specified. Otherwise an exception
          will occurr upon timer expiration:
          ``TypeError: 'NoneType' object isn't callable``
      """
    def deinit(self) -> None:
        """
      Deinitialises the timer. Stops the timer, and disables the timer peripheral.
      """

class WDT:
    """
   The WDT is used to restart the system when the application crashes and ends
   up into a non recoverable state. Once started it cannot be stopped or
   reconfigured in any way. After enabling, the application must "feed" the
   watchdog periodically to prevent it from expiring and resetting the system.
   
   Example usage::
   
       from machine import WDT
       wdt = WDT(timeout=2000)  # enable it with a timeout of 2s
       wdt.feed()
   
   Availability of this class: pyboard, WiPy, esp8266, esp32.
   """

    def __init__(self, *, id: int = 0, timeout: int = 5000):
        """
      Create a WDT object and start it. The timeout must be given in milliseconds.
      Once it is running the timeout cannot be changed and the WDT cannot be stopped either.
      
      Notes: On the esp32 the minimum timeout is 1 second. On the esp8266 a timeout
      cannot be specified, it is determined by the underlying system.
      """
    def feed(self) -> None:
        """
      Feed the WDT to prevent it from resetting the system. The application
      should place this call in a sensible place ensuring that the WDT is
      only fed after verifying that everything is functioning correctly.
      """

class SD:
    """
   .. warning::
   
      This is a non-standard class and is only available on the cc3200 port.
   
   
   The SD card class allows to configure and enable the memory card
   module of the WiPy and automatically mount it as ``/sd`` as part
   of the file system. There are several pin combinations that can be
   used to wire the SD card socket to the WiPy and the pins used can
   be specified in the constructor. Please check the `pinout and alternate functions
   table. <https://raw.githubusercontent.com/wipy/wipy/master/docs/PinOUT.png>`_ for
   more info regarding the pins which can be remapped to be used with a SD card.
   
   Example usage::
   
       from machine import SD
       import os
       # clk cmd and dat0 pins must be passed along with
       # their respective alternate functions
       sd = machine.SD(pins=('GP10', 'GP11', 'GP15'))
       os.mount(sd, '/sd')
       # do normal file operations
   """

    def __init__(
        self,
        id: int = 0,
        pins: tuple[str, str, str] | tuple[Pin, Pin, Pin] = ("GP10", "GP11", "GP15"),
        /,
    ):
        """
      Create a SD card object. See ``init()`` for parameters if initialization.
      """
    def init(
        self,
        id: int = 0,
        pins: tuple[str, str, str] | tuple[Pin, Pin, Pin] = ("GP10", "GP11", "GP15"),
        /,
    ) -> None:
        """
      Enable the SD card. In order to initialize the card, give it a 3-tuple:
      ``(clk_pin, cmd_pin, dat0_pin)``.
      """
    def deinit(self) -> None:
        """
      Disable the SD card.
      """

# noinspection PyShadowingNames
class SDCard(AbstractBlockDev):
    """
   SD cards are one of the most common small form factor removable storage media.
   SD cards come in a variety of sizes and physical form factors. MMC cards are
   similar removable storage devices while eMMC devices are electrically similar
   storage devices designed to be embedded into other systems. All three form
   share a common protocol for communication with their host system and high-level
   support looks the same for them all. As such in MicroPython they are implemented
   in a single class called :class:`machine.SDCard` .
   
   Both SD and MMC interfaces support being accessed with a variety of bus widths.
   When being accessed with a 1-bit wide interface they can be accessed using the
   SPI protocol. Different MicroPython hardware platforms support different widths
   and pin configurations but for most platforms there is a standard configuration
   for any given hardware. In general constructing an ``SDCard`` object with without
   passing any parameters will initialise the interface to the default card slot
   for the current hardware. The arguments listed below represent the common
   arguments that might need to be set in order to use either a non-standard slot
   or a non-standard pin assignment. The exact subset of arguments supported will
   vary from platform to platform.
   

   Implementation-specific details
   -------------------------------
   
   Different implementations of the ``SDCard`` class on different hardware support
   varying subsets of the options above.
   
   PyBoard
   ```````
   
   The standard PyBoard has just one slot. No arguments are necessary or supported.
   
   ESP32
   `````
   
   The ESP32 provides two channels of SD/MMC hardware and also supports
   access to SD Cards through either of the two SPI ports that are
   generally available to the user. As a result the *slot* argument can
   take a value between 0 and 3, inclusive. Slots 0 and 1 use the
   built-in SD/MMC hardware while slots 2 and 3 use the SPI ports. Slot 0
   supports 1, 4 or 8-bit wide access while slot 1 supports 1 or 4-bit
   access; the SPI slots only support 1-bit access.
   
     .. note:: Slot 0 is used to communicate with on-board flash memory
               on most ESP32 modules and so will be unavailable to the
               user.
   
     .. note:: Most ESP32 modules that provide an SD card slot using the
               dedicated hardware only wire up 1 data pin, so the default
               value for *width* is 1.
   
   The pins used by the dedicated SD/MMC hardware are fixed. The pins
   used by the SPI hardware can be reassigned.
   
     .. note:: If any of the SPI signals are remapped then all of the SPI
               signals will pass through a GPIO multiplexer unit which
               can limit the performance of high frequency signals. Since
               the normal operating speed for SD cards is 40MHz this can
               cause problems on some cards.
   
   The default (and preferred) pin assignment are as follows:
   
       ====== ====== ====== ====== ======
       Slot   0      1      2      3
       ------ ------ ------ ------ ------
       Signal   Pin    Pin    Pin    Pin
       ====== ====== ====== ====== ======
       sck       6     14     18     14
       cmd      11     15
       cs                      5     15
       miso                   19     12
       mosi                   23     13
       D0        7      2
       D1        8      4
       D2        9     12
       D3       10     13
       D4       16
       D5       17
       D6        5
       D7       18
       ====== ====== ====== ====== ======
   
   cc3200
   ``````
   
   You can set the pins used for SPI access by passing a tuple as the
   *pins* argument.
   
   *Note:* The current cc3200 SD card implementation names the this class
   :class:`machine.SD` rather than :class:`machine.SDCard` .
   """

    def __init__(
        self,
        slot: int = 1,
        width: int = 1,
        cd: int | str | Pin | None = None,
        wp: int | str | Pin | None = None,
        sck: int | str | Pin | None = None,
        miso: int | str | Pin | None = None,
        mosi: int | str | Pin | None = None,
        cs: int | str | Pin | None = None,
        freq: int = 20000000,
        /,
    ):
        """
       This class provides access to SD or MMC storage cards using either
       a dedicated SD/MMC interface hardware or through an SPI channel.
       The class implements the block protocol defined by :class:`os.AbstractBlockDev`.
       This allows the mounting of an SD card to be as simple as::
       
         os.mount(machine.SDCard(), "/sd")
       
       The constructor takes the following parameters:
       
        - *slot* selects which of the available interfaces to use. Leaving this
          unset will select the default interface.
       
        - *width* selects the bus width for the SD/MMC interface.
       
        - *cd* can be used to specify a card-detect pin.
       
        - *wp* can be used to specify a write-protect pin.
       
        - *sck* can be used to specify an SPI clock pin.
       
        - *miso* can be used to specify an SPI miso pin.
       
        - *mosi* can be used to specify an SPI mosi pin.
       
        - *cs* can be used to specify an SPI chip select pin.
        
        - *freq* selects the SD/MMC interface frequency in Hz (only supported on the ESP32).
      """



import cv2
import platform

# 获取当前操作系统
current_os = platform.system()

if current_os == "Windows":
    save_path = "D:/copy.png"
else:  # 默认为Ubuntu或其他Linux发行版
    save_path = "/home/username/copy.png"
cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)  # 打开笔记本内置摄像头
while cap.isOpened():  # 笔记本内置摄像头被打开后
    ret, frame = cap.read()  # 从摄像头中实时读取视频
    cv2.imshow("Video", frame)  # 在窗口中显示视频
    k = cv2.waitKey(1)  # 图像的刷新时间为1毫秒
    if k == ord(' '):  # 按下空格键
        cv2.imshow('img', frame)  # 显示按下空格键时摄像头视频中的图像
        cv2.imwrite(save_path, frame)  # 保存按下空格键时摄像头视频中的图像
        cap.release()  # 关闭笔记本内置摄像头
        cv2.destroyWindow("Video")  # 销毁名为Video的窗口
        break
cv2.destroyAllWindows()  # 销毁显示图像的窗口



"""
mathematical functions.

Descriptions taken from:
https://raw.githubusercontent.com/micropython/micropython/master/docs/library/math.rst.
=====================================

.. module:: math
   :synopsis: mathematical functions

|see_cpython_module| :mod:`python:math`.

The ``math`` module provides some basic mathematical functions for
working with floating-point numbers.

*Note:* On the pyboard, floating-point numbers have 32-bit precision.

Availability: not available on WiPy. Floating point support required
for this module.
"""

__author__ = "Howard C Lovatt"
__copyright__ = "Howard C Lovatt, 2020 onwards."
__license__ = "MIT https://opensource.org/licenses/MIT (as used by MicroPython)."
__version__ = "7.3.9"  # Version set by https://github.com/hlovatt/tag2ver

from typing import SupportsFloat, Final

def acos(x: SupportsFloat, /) -> float:
    """
   Return the inverse cosine of ``x``.
   """

def acosh(x: SupportsFloat, /) -> float:
    """
   Return the inverse hyperbolic cosine of ``x``.
   """

def asin(x: SupportsFloat, /) -> float:
    """
   Return the inverse sine of ``x``.
   """

def asinh(x: SupportsFloat, /) -> float:
    """
   Return the inverse hyperbolic sine of ``x``.
   """

def atan(x: SupportsFloat, /) -> float:
    """
   Return the inverse tangent of ``x``.
   """

def atan2(y: SupportsFloat, x: SupportsFloat, /) -> float:
    """
   Return the principal value of the inverse tangent of ``y/x``.
   """

def atanh(x: SupportsFloat, /) -> float:
    """
   Return the inverse hyperbolic tangent of ``x``.
   """

def ceil(x: SupportsFloat, /) -> int:
    """
   Return an integer, being ``x`` rounded towards positive infinity.
   """

def copysign(x: SupportsFloat, y: SupportsFloat, /) -> float:
    """
   Return ``x`` with the sign of ``y``.
   """

def cos(x: SupportsFloat, /) -> float:
    """
   Return the cosine of ``x``.
   """

def cosh(x: SupportsFloat, /) -> float:
    """
   Return the hyperbolic cosine of ``x``.
   """

def degrees(x: SupportsFloat, /) -> float:
    """
   Return radians ``x`` converted to degrees.
   """

def erf(x: SupportsFloat, /) -> float:
    """
   Return the error function of ``x``.
   """

def erfc(x: SupportsFloat, /) -> float:
    """
   Return the complementary error function of ``x``.
   """

def exp(x: SupportsFloat, /) -> float:
    """
   Return the exponential of ``x``.
   """

def expm1(x: SupportsFloat, /) -> float:
    """
   Return ``exp(x) - 1``.
   """

def fabs(x: SupportsFloat, /) -> float:
    """
   Return the absolute value of ``x``.
   """

def floor(x: SupportsFloat, /) -> int:
    """
   Return an integer, being ``x`` rounded towards negative infinity.
   """

def fmod(x: SupportsFloat, y: SupportsFloat, /) -> float:
    """
   Return the remainder of ``x/y``.
   """

def frexp(x: SupportsFloat, /) -> tuple[float, int]:
    """
   Decomposes a floating-point number into its mantissa and exponent.
   The returned value is the tuple ``(m, e)`` such that ``x == m * 2**e``
   exactly.  If ``x == 0`` then the function returns ``(0.0, 0)``, otherwise
   the relation ``0.5 <= abs(m) < 1`` holds.
   """

def gamma(x: SupportsFloat, /) -> float:
    """
   Return the gamma function of ``x``.
   """

def isfinite(x: SupportsFloat, /) -> bool:
    """
   Return ``True`` if ``x`` is finite.
   """

def isinf(x: SupportsFloat, /) -> bool:
    """
   Return ``True`` if ``x`` is infinite.
   """

def isnan(x: SupportsFloat, /) -> bool:
    """
   Return ``True`` if ``x`` is not-a-number
   """

# noinspection PyShadowingNames
def ldexp(x: SupportsFloat, exp: int, /) -> float:
    """
   Return ``x * (2**exp)``.
   """

def lgamma(x: SupportsFloat, /) -> float:
    """
   Return the natural logarithm of the gamma function of ``x``.
   """

def log(x: SupportsFloat, /) -> float:
    """
   Return the natural logarithm of ``x``.
   """

def log10(x: SupportsFloat, /) -> float:
    """
   Return the base-10 logarithm of ``x``.
   """

def log2(x: SupportsFloat, /) -> float:
    """
   Return the base-2 logarithm of ``x``.
   """

def modf(x: SupportsFloat, /) -> tuple[float, float]:
    """
   Return a tuple of two floats, being the fractional and integral parts of
   ``x``.  Both return values have the same sign as ``x``.
   """

def pow(x: SupportsFloat, y: SupportsFloat, /) -> float:
    """
   Returns ``x`` to the power of ``y``.
   """

def radians(x: SupportsFloat, /) -> float:
    """
   Return degrees ``x`` converted to radians.
   """

def sin(x: SupportsFloat, /) -> float:
    """
   Return the sine of ``x``.
   """

def sinh(x: SupportsFloat, /) -> float:
    """
   Return the hyperbolic sine of ``x``.
   """

def sqrt(x: SupportsFloat, /) -> float:
    """
   Return the square root of ``x``.
   """

def tan(x: SupportsFloat, /) -> float:
    """
   Return the tangent of ``x``.
   """

def tanh(x: SupportsFloat, /) -> float:
    """
   Return the hyperbolic tangent of ``x``.
   """

def trunc(x: SupportsFloat, /) -> float:
    """
   Return an integer, being ``x`` rounded towards 0.
   """

e: Final[float] = ...
"""
base of the natural logarithm
"""

pi: Final[float] = ...
"""
the ratio of a circle's circumference to its diameter
"""
